@@ -15,13 +15,13 @@ Functions:
 qblast        Do a BLAST search using the QBLAST API.
 """
 
-import sys
 try:
     from cStringIO import StringIO
 except ImportError:
     from StringIO import StringIO
 
-from Bio._py3k import _as_string
+from Bio._py3k import _as_string, _as_bytes
+
 
 def qblast(program, database, sequence,
            auto_format=None,composition_based_statistics=None,
@@ -36,7 +36,7 @@ def qblast(program, database, sequence,
            alignments=500,alignment_view=None,descriptions=500,
            entrez_links_new_window=None,expect_low=None,expect_high=None,
            format_entrez_query=None,format_object=None,format_type='XML',
-           ncbi_gi=None,results_file=None,show_overview=None
+           ncbi_gi=None,results_file=None,show_overview=None, megablast=None,
            ):
     """Do a BLAST search using the QBLAST server at NCBI.
 
@@ -54,19 +54,25 @@ def qblast(program, database, sequence,
     format_type    "HTML", "Text", "ASN.1", or "XML".  Def. "XML".
     entrez_query   Entrez query to limit Blast search
     hitlist_size   Number of hits to return. Default 50
+    megablast      TRUE/FALSE whether to use MEga BLAST algorithm (blastn only)
+    service        plain, psi, phi, rpsblast, megablast (lower case)
 
     This function does no checking of the validity of the parameters
     and passes the values to the server as is.  More help is available at:
-    http://www.ncbi.nlm.nih.gov/BLAST/blast_overview.html
+    http://www.ncbi.nlm.nih.gov/BLAST/Doc/urlapi.html
 
     """
-    import urllib, urllib2
+    import urllib
+    import urllib2
     import time
 
     assert program in ['blastn', 'blastp', 'blastx', 'tblastn', 'tblastx']
 
     # Format the "Put" command, which sends search requests to qblast.
     # Parameters taken from http://www.ncbi.nlm.nih.gov/BLAST/Doc/node5.html on 9 July 2007
+    # Additional parameters are taken from http://www.ncbi.nlm.nih.gov/BLAST/Doc/node9.html on 8 Oct 2010
+    # To perform a PSI-BLAST or PHI-BLAST search the service ("Put" and "Get" commands) must be specified
+    # (e.g. psi_blast = NCBIWWW.qblast("blastp", "refseq_protein", input_sequence, service="psi"))
     parameters = [
         ('AUTO_FORMAT',auto_format),
         ('COMPOSITION_BASED_STATISTICS',composition_based_statistics),
@@ -82,6 +88,7 @@ def qblast(program, database, sequence,
         ('I_THRESH',i_thresh),
         ('LAYOUT',layout),
         ('LCASE_MASK',lcase_mask),
+        ('MEGABLAST',megablast),
         ('MATRIX_NAME',matrix_name),
         ('NUCL_PENALTY',nucl_penalty),
         ('NUCL_REWARD',nucl_reward),
@@ -89,11 +96,13 @@ def qblast(program, database, sequence,
         ('PERC_IDENT',perc_ident),
         ('PHI_PATTERN',phi_pattern),
         ('PROGRAM',program),
+        #('PSSM',pssm), - It is possible to use PSI-BLAST via this API?
         ('QUERY',sequence),
         ('QUERY_FILE',query_file),
         ('QUERY_BELIEVE_DEFLINE',query_believe_defline),
         ('QUERY_FROM',query_from),
         ('QUERY_TO',query_to),
+        #('RESULTS_FILE',...), - Can we use this parameter?
         ('SEARCHSP_EFF',searchsp_eff),
         ('SERVICE',service),
         ('THRESHOLD',threshold),
@@ -102,7 +111,7 @@ def qblast(program, database, sequence,
         ('CMD', 'Put'),
         ]
     query = [x for x in parameters if x[1] is not None]
-    message = urllib.urlencode(query)
+    message = _as_bytes(urllib.urlencode(query))
 
     # Send off the initial query to qblast.
     # Note the NCBI do not currently impose a rate limit here, other
@@ -114,7 +123,7 @@ def qblast(program, database, sequence,
     handle = urllib2.urlopen(request)
 
     # Format the "Get" command, which gets the formatted results from qblast
-    # Parameters taken from http://www.ncbi.nlm.nih.gov/BLAST/Doc/node6.html on 9 July 2007    
+    # Parameters taken from http://www.ncbi.nlm.nih.gov/BLAST/Doc/node6.html on 9 July 2007
     rid, rtoe = _parse_qblast_ref_page(handle)
     parameters = [
         ('ALIGNMENTS',alignments),
@@ -134,7 +143,7 @@ def qblast(program, database, sequence,
         ('CMD', 'Get'),
         ]
     query = [x for x in parameters if x[1] is not None]
-    message = urllib.urlencode(query)
+    message = _as_bytes(urllib.urlencode(query))
 
     # Poll NCBI until the results are ready.  Use a 3 second wait
     delay = 3.0
@@ -159,7 +168,7 @@ def qblast(program, database, sequence,
         if results=="\n\n":
             continue
         # XML results don't have the Status tag when finished
-        if results.find("Status=") < 0:
+        if "Status=" not in results:
             break
         i = results.index("Status=")
         j = results.index("\n", i)
@@ -169,6 +178,7 @@ def qblast(program, database, sequence,
 
     return StringIO(results)
 
+
 def _parse_qblast_ref_page(handle):
     """Extract a tuple of RID, RTOE from the 'please wait' page (PRIVATE).
 
@@ -234,7 +244,5 @@ def _parse_qblast_ref_page(handle):
     try:
         return rid, int(rtoe)
     except ValueError:
-        raise ValueError("A non-integer RTOE found in " \
+        raise ValueError("A non-integer RTOE found in "
                          +"the 'please wait' page, %s" % repr(rtoe))
-
-    
