@@ -40,16 +40,41 @@ espell       Retrieves spelling suggestions.
 
 read         Parses the XML results returned by any of the above functions.
              Typical usage is:
+
+             >>> from Bio import Entrez
+             >>> Entrez.email = "Your.Name.Here@example.org"
              >>> handle = Entrez.einfo() # or esearch, efetch, ...
              >>> record = Entrez.read(handle)
+             >>> handle.close()
+
              where record is now a Python dictionary or list.
 
+parse        Parses the XML results returned by those of the above functions
+             which can return multiple records - such as efetch, esummary
+             and elink. Typical usage is:
+
+             >>> handle = Entrez.efetch("pubmed", id="19304878,14630660", retmode="xml")
+             >>> records = Entrez.parse(handle)
+             >>> for record in records:
+             ...     # each record is a Python dictionary or list.
+             ...     print record['MedlineCitation']['Article']['ArticleTitle']
+             Biopython: freely available Python tools for computational molecular biology and bioinformatics.
+             PDB file parser and structure class implemented in Python.
+             >>> handle.close()
+
+             This function is appropriate only if the XML file contains
+             multiple records, and is particular useful for large files.
+
 _open        Internally used function.
 
 """
-import urllib, urllib2, time, warnings
+import urllib
+import urllib2
+import time
+import warnings
 import os.path
 
+from Bio._py3k import _binary_to_string_handle, _as_bytes
 
 email = None
 tool = "biopython"
@@ -69,12 +94,13 @@ def epost(db, **keywds):
 
     Raises an IOError exception if there's a network error.
     """
-    cgi='http://eutils.ncbi.nlm.nih.gov/entrez/eutils/epost.fcgi'
-    variables = {'db' : db}
+    cgi = 'http://eutils.ncbi.nlm.nih.gov/entrez/eutils/epost.fcgi'
+    variables = {'db': db}
     variables.update(keywds)
     return _open(cgi, variables, post=True)
 
-def efetch(db, **keywds):
+
+def efetch(db, **keywords):
     """Fetches Entrez results which are returned as a handle.
 
     EFetch retrieves records in the requested format from a list of one or
@@ -89,14 +115,34 @@ def efetch(db, **keywds):
 
     Short example:
 
-    from Bio import Entrez
-    handle = Entrez.efetch(db="nucleotide", id="57240072", rettype="gb")
-    print handle.read()
+    >>> from Bio import Entrez
+    >>> Entrez.email = "Your.Name.Here@example.org"
+    >>> handle = Entrez.efetch(db="nucleotide", id="57240072", rettype="gb", retmode="text")
+    >>> print handle.readline().strip()
+    LOCUS       AY851612                 892 bp    DNA     linear   PLN 10-APR-2007
+    >>> handle.close()
+
+    Warning: The NCBI changed the default retmode in Feb 2012, so many
+    databases which previously returned text output now give XML.
     """
-    cgi='http://eutils.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi'
-    variables = {'db' : db}
-    variables.update(keywds)
-    return _open(cgi, variables)
+    cgi = 'http://eutils.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi'
+    variables = {'db': db}
+    variables.update(keywords)
+    post = False
+    try:
+        ids = variables["id"]
+    except KeyError:
+        pass
+    else:
+        if isinstance(ids, list):
+            ids = ",".join(ids)
+            variables["id"] = ids
+        if ids.count(",") >= 200:
+            # NCBI prefers an HTTP POST instead of an HTTP GET if there are
+            # more than about 200 IDs
+            post = True
+    return _open(cgi, variables, post)
+
 
 def esearch(db, term, **keywds):
     """ESearch runs an Entrez search and returns a handle to the results.
@@ -114,18 +160,26 @@ def esearch(db, term, **keywds):
 
     Short example:
 
-    from Bio import Entez
-    handle = Entrez.esearch(db="nucleotide", retmax=10, term="Opuntia")
-    record = Entrez.read(handle)
-    print record["Count"]
-    print record["IdList"]
+    >>> from Bio import Entrez
+    >>> Entrez.email = "Your.Name.Here@example.org"
+    >>> handle = Entrez.esearch(db="nucleotide", retmax=10, term="opuntia[ORGN] accD")
+    >>> record = Entrez.read(handle)
+    >>> handle.close()
+    >>> record["Count"] >= 2
+    True
+    >>> "156535671" in record["IdList"]
+    True
+    >>> "156535673" in record["IdList"]
+    True
+
     """
-    cgi='http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi'
-    variables = {'db' : db,
-                 'term' : term}
+    cgi = 'http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi'
+    variables = {'db': db,
+                 'term': term}
     variables.update(keywds)
     return _open(cgi, variables)
 
+
 def elink(**keywds):
     """ELink checks for linked external articles and returns a handle.
 
@@ -141,12 +195,30 @@ def elink(**keywds):
     Return a handle to the results, by default in XML format.
 
     Raises an IOError exception if there's a network error.
+
+    This example finds articles related to the Biopython application
+    note's entry in the PubMed database:
+
+    >>> from Bio import Entrez
+    >>> Entrez.email = "Your.Name.Here@example.org"
+    >>> pmid = "19304878"
+    >>> handle = Entrez.elink(dbfrom="pubmed", id=pmid, linkname="pubmed_pubmed")
+    >>> record = Entrez.read(handle)
+    >>> handle.close()
+    >>> print record[0]["LinkSetDb"][0]["LinkName"]
+    pubmed_pubmed
+    >>> linked = [link["Id"] for link in record[0]["LinkSetDb"][0]["Link"]]
+    >>> "17121776" in linked
+    True
+
+    This is explained in much more detail in the Biopython Tutorial.
     """
-    cgi='http://eutils.ncbi.nlm.nih.gov/entrez/eutils/elink.fcgi'
+    cgi = 'http://eutils.ncbi.nlm.nih.gov/entrez/eutils/elink.fcgi'
     variables = {}
     variables.update(keywds)
     return _open(cgi, variables)
 
+
 def einfo(**keywds):
     """EInfo returns a summary of the Entez databases as a results handle.
 
@@ -162,15 +234,19 @@ def einfo(**keywds):
 
     Short example:
 
-    from Bio import Entrez 
-    record = Entrez.read(Entrez.einfo())
-    print record['DbList']
+    >>> from Bio import Entrez
+    >>> Entrez.email = "Your.Name.Here@example.org"
+    >>> record = Entrez.read(Entrez.einfo())
+    >>> 'pubmed' in record['DbList']
+    True
+
     """
-    cgi='http://eutils.ncbi.nlm.nih.gov/entrez/eutils/einfo.fcgi'
+    cgi = 'http://eutils.ncbi.nlm.nih.gov/entrez/eutils/einfo.fcgi'
     variables = {}
     variables.update(keywds)
     return _open(cgi, variables)
 
+
 def esummary(**keywds):
     """ESummary retrieves document summaries as a results handle.
 
@@ -183,12 +259,26 @@ def esummary(**keywds):
     Return a handle to the results, by default in XML format.
 
     Raises an IOError exception if there's a network error.
+
+    This example discovers more about entry 30367 in the journals database:
+
+    >>> from Bio import Entrez
+    >>> Entrez.email = "Your.Name.Here@example.org"
+    >>> handle = Entrez.esummary(db="journals", id="30367")
+    >>> record = Entrez.read(handle)
+    >>> handle.close()
+    >>> print record[0]["Id"]
+    30367
+    >>> print record[0]["Title"]
+    Computational biology and chemistry
+
     """
-    cgi='http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esummary.fcgi'
+    cgi = 'http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esummary.fcgi'
     variables = {}
     variables.update(keywds)
     return _open(cgi, variables)
 
+
 def egquery(**keywds):
     """EGQuery provides Entrez database counts for a global search.
 
@@ -201,12 +291,28 @@ def egquery(**keywds):
     Return a handle to the results in XML format.
 
     Raises an IOError exception if there's a network error.
+
+    This quick example based on a longer version from the Biopython
+    Tutorial just checks there are over 60 matches for 'Biopython'
+    in PubMedCentral:
+
+    >>> from Bio import Entrez
+    >>> Entrez.email = "Your.Name.Here@example.org"
+    >>> handle = Entrez.egquery(term="biopython")
+    >>> record = Entrez.read(handle)
+    >>> handle.close()
+    >>> for row in record["eGQueryResult"]:
+    ...     if "pmc" in row["DbName"]:
+    ...         print row["Count"] > 60
+    True
+
     """
-    cgi='http://eutils.ncbi.nlm.nih.gov/entrez/eutils/egquery.fcgi'
+    cgi = 'http://eutils.ncbi.nlm.nih.gov/entrez/eutils/egquery.fcgi'
     variables = {}
     variables.update(keywds)
     return _open(cgi, variables)
 
+
 def espell(**keywds):
     """ESpell retrieves spelling suggestions, returned in a results handle.
 
@@ -221,25 +327,35 @@ def espell(**keywds):
 
     Short example:
 
-    from Bio import Entrez 
-    record = Entrez.read(Entrez.espell(term="biopythooon")) 
-    print record["Query"] 
-    print record["CorrectedQuery"] 
+    >>> from Bio import Entrez
+    >>> Entrez.email = "Your.Name.Here@example.org"
+    >>> record = Entrez.read(Entrez.espell(term="biopythooon"))
+    >>> print record["Query"]
+    biopythooon
+    >>> print record["CorrectedQuery"]
+    biopython
+
     """
-    cgi='http://eutils.ncbi.nlm.nih.gov/entrez/eutils/espell.fcgi'
+    cgi = 'http://eutils.ncbi.nlm.nih.gov/entrez/eutils/espell.fcgi'
     variables = {}
     variables.update(keywds)
     return _open(cgi, variables)
 
-def read(handle):
+
+def read(handle, validate=True):
     """Parses an XML file from the NCBI Entrez Utilities into python objects.
-    
+
     This function parses an XML file created by NCBI's Entrez Utilities,
     returning a multilevel data structure of Python lists and dictionaries.
     Most XML files returned by NCBI's Entrez Utilities can be parsed by
     this function, provided its DTD is available. Biopython includes the
     DTDs for most commonly used Entrez Utilities.
 
+    If validate is True (default), the parser will validate the XML file
+    against the DTD, and raise an error if the XML file contains tags that
+    are not represented in the DTD. If validate is False, the parser will
+    simply skip such tags.
+
     Whereas the data structure seems to consist of generic Python lists,
     dictionaries, strings, and so on, each of these is actually a class
     derived from the base type. This allows us to store the attributes
@@ -247,18 +363,43 @@ def read(handle):
     the tag name in my_element.tag.
     """
     from Parser import DataHandler
-    DTDs = os.path.join(str(__path__[0]), "DTDs")
-    handler = DataHandler(DTDs)
+    handler = DataHandler(validate)
     record = handler.read(handle)
     return record
 
-def parse(handle):
+
+def parse(handle, validate=True):
+    """Parses an XML file from the NCBI Entrez Utilities into python objects.
+
+    This function parses an XML file created by NCBI's Entrez Utilities,
+    returning a multilevel data structure of Python lists and dictionaries.
+    This function is suitable for XML files that (in Python) can be represented
+    as a list of individual records. Whereas 'read' reads the complete file
+    and returns a single Python list, 'parse' is a generator function that
+    returns the records one by one. This function is therefore particularly
+    useful for parsing large files.
+
+    Most XML files returned by NCBI's Entrez Utilities can be parsed by
+    this function, provided its DTD is available. Biopython includes the
+    DTDs for most commonly used Entrez Utilities.
+
+    If validate is True (default), the parser will validate the XML file
+    against the DTD, and raise an error if the XML file contains tags that
+    are not represented in the DTD. If validate is False, the parser will
+    simply skip such tags.
+
+    Whereas the data structure seems to consist of generic Python lists,
+    dictionaries, strings, and so on, each of these is actually a class
+    derived from the base type. This allows us to store the attributes
+    (if any) of each element in a dictionary my_element.attributes, and
+    the tag name in my_element.tag.
+    """
     from Parser import DataHandler
-    DTDs = os.path.join(str(__path__[0]), "DTDs")
-    handler = DataHandler(DTDs)
+    handler = DataHandler(validate)
     records = handler.parse(handle)
     return records
 
+
 def _open(cgi, params={}, post=False):
     """Helper function to build the URL and open a handle to it (PRIVATE).
 
@@ -289,7 +430,7 @@ def _open(cgi, params={}, post=False):
         params["tool"] = tool
     # Tell Entrez who we are
     if not "email" in params:
-        if email!=None:
+        if email is not None:
             params["email"] = email
         else:
             warnings.warn("""
@@ -306,10 +447,11 @@ a user at the email address provided before blocking access to the
 E-utilities.""", UserWarning)
     # Open a handle to Entrez.
     options = urllib.urlencode(params, doseq=True)
+    #print cgi + "?" + options
     try:
         if post:
             #HTTP POST
-            handle = urllib2.urlopen(cgi, data=options)
+            handle = urllib2.urlopen(cgi, data=_as_bytes(options))
         else:
             #HTTP GET
             cgi += "?" + options
@@ -317,6 +459,17 @@ E-utilities.""", UserWarning)
     except urllib2.HTTPError, exception:
         raise exception
 
-    return handle
+    return _binary_to_string_handle(handle)
 
 _open.previous = 0
+
+
+def _test():
+    """Run the module's doctests (PRIVATE)."""
+    print "Running doctests..."
+    import doctest
+    doctest.testmod()
+    print "Done"
+
+if __name__ == "__main__":
+    _test()
