@@ -32,12 +32,14 @@ from reportlab.lib import colors
 
 # GenomeDiagram imports
 from _AbstractDrawer import AbstractDrawer, draw_box, draw_arrow
+from _AbstractDrawer import draw_cut_corner_box, _stroke_and_fill_colors
 from _AbstractDrawer import intermediate_points, angle2trig
 from _FeatureSet import FeatureSet
 from _GraphSet import GraphSet
 
 from math import ceil
 
+
 class LinearDrawer(AbstractDrawer):
     """ LinearDrawer(AbstractDrawer)
 
@@ -73,7 +75,7 @@ class LinearDrawer(AbstractDrawer):
 
         o set_track_heights(self)   Calculate information about the offset of
                                     each track from the fragment base
-                                    
+
         o draw_test_tracks(self)    Add lines demarcating each track to the
                                     drawing
 
@@ -172,7 +174,7 @@ class LinearDrawer(AbstractDrawer):
         o track_offsets     Dictionary of number of pixels that each track top,
                             center and bottom is offset from the base of a
                             fragment, keyed by track
-        
+
         o cross_track_links List of tuples each with four entries (track A,
                             feature A, track B, feature B) to be linked.
 
@@ -221,14 +223,14 @@ class LinearDrawer(AbstractDrawer):
             o end       Int, the position to stop drawing the diagram at
 
             o tracklines    Boolean flag to show (or not) lines delineating tracks
-                            on the diagram            
+                            on the diagram
 
             o fragments Int, the number of equal fragments into which the
                         sequence should be divided for drawing
 
             o fragment_size Float(0->1) The proportion of the available height
                             for the fragment that should be taken up in drawing
-                            
+
             o track_size    The proportion of the available track height that
                             should be taken up in drawing
 
@@ -245,7 +247,6 @@ class LinearDrawer(AbstractDrawer):
         self.fragment_size = fragment_size
         self.track_size = track_size
 
-
     def draw(self):
         """ draw(self)
 
@@ -304,11 +305,10 @@ class LinearDrawer(AbstractDrawer):
         for element_group in element_groups:
             for element_list in element_group:
                 [self.drawing.add(element) for element in element_list]
-            
+
         if self.tracklines:             # Draw test tracks over top of diagram
             self.draw_test_tracks()
 
-
     def init_fragments(self):
         """ init_fragments(self)
 
@@ -338,7 +338,6 @@ class LinearDrawer(AbstractDrawer):
             self.fragment_limits[fragment_count] = (marker, marker+fragment_step)
             fragment_count += 1
 
-
     def set_track_heights(self):
         """ set_track_heights(self)
 
@@ -371,7 +370,7 @@ class LinearDrawer(AbstractDrawer):
             top = trackunits[track][1]*trackunit_height - track_crop  # top offset
             btm = trackunits[track][0]*trackunit_height + track_crop  # bottom offset
             ctr = btm+(top-btm)/2.                          # center offset
-            track_offsets[track] = (btm, ctr, top)          
+            track_offsets[track] = (btm, ctr, top)
         self.track_offsets = track_offsets
 
     def draw_test_tracks(self):
@@ -409,18 +408,17 @@ class LinearDrawer(AbstractDrawer):
         """
         track_elements = []     # Holds elements from features and graphs
         track_labels = []       # Holds labels from features and graphs
-        
+
         # Distribution dictionary for dealing with different set types
         set_methods = {FeatureSet: self.draw_feature_set,
                        GraphSet: self.draw_graph_set
                        }
-        
+
         for set in track.get_sets():        # Draw the feature or graph sets
             elements, labels = set_methods[set.__class__](set)
             track_elements += elements
             track_labels += labels
         return track_elements, track_labels
-        
 
     def draw_tick(self, tickpos, ctr, ticklen, track, draw_label):
         """ draw_tick(self, tickpos, ctr, ticklen) -> (element, element)
@@ -561,7 +559,7 @@ class LinearDrawer(AbstractDrawer):
             for set in track.get_sets():            # Check all sets...
                 if set.__class__ is GraphSet:     # ...for a graph set
                     graph_label_min = []
-                    graph_label_mid = []                    
+                    graph_label_mid = []
                     graph_label_max = []
                     for graph in set.get_graphs():
                         quartiles = graph.quartiles()
@@ -601,8 +599,6 @@ class LinearDrawer(AbstractDrawer):
                             scale_labels.append(labelgroup)
 
         return scale_elements, scale_labels
-        
-        
 
     def draw_greytrack(self, track):
         """ draw_greytrack(self) -> ([element, element,...], [element, element,...])
@@ -611,7 +607,7 @@ class LinearDrawer(AbstractDrawer):
 
             Put in a grey background to the current track in all fragments,
             if track specifies that we should
-        """        
+        """
         greytrack_bgs = []      # Holds grey track backgrounds
         greytrack_labels = []   # Holds grey foreground labels
 
@@ -663,7 +659,6 @@ class LinearDrawer(AbstractDrawer):
 
         return greytrack_bgs, greytrack_labels
 
-
     def draw_feature_set(self, set):
         """ draw_feature_set(self, set) -> ([element, element,...], [element, element,...])
 
@@ -674,7 +669,7 @@ class LinearDrawer(AbstractDrawer):
         """
         #print 'draw feature set'
         feature_elements = []   # Holds diagram elements belonging to the features
-        label_elements = []     # Holds diagram elements belonging to feature labels 
+        label_elements = []     # Holds diagram elements belonging to feature labels
 
         # Collect all the elements for the feature set
         for feature in set.get_features():
@@ -695,7 +690,7 @@ class LinearDrawer(AbstractDrawer):
         """
         if feature.hide:        # Feature hidden, don't draw it...
             return [], []
-        
+
         feature_elements = []   # Holds diagram elements belonging to the feature
         label_elements = []     # Holds labels belonging to the feature
 
@@ -713,7 +708,7 @@ class LinearDrawer(AbstractDrawer):
                 feature_elements.append(box)
                 if label is not None:
                     label_elements.append(label)
-            
+
         return feature_elements, label_elements
 
     def draw_feature_location(self, feature, locstart, locend):
@@ -731,7 +726,6 @@ class LinearDrawer(AbstractDrawer):
         # end fragments do actually exist in terms of the drawing
         allowed_fragments = self.fragment_limits.keys()
         if start_fragment in allowed_fragments and end_fragment in allowed_fragments:
-            
             #print feature.name, feature.start, feature.end, start_offset, end_offset
             if start_fragment == end_fragment:  # Feature is found on one fragment
                 feature_box, label = self.get_feature_sigil(feature, start_offset,
@@ -739,7 +733,7 @@ class LinearDrawer(AbstractDrawer):
                 feature_boxes.append((feature_box, label))
                 #feature_elements.append(feature_box)
                 #if label is not None:   # There is a label for the feature
-                #    label_elements.append(label)                        
+                #    label_elements.append(label)
             else: # Feature is split over two or more fragments
                 fragment = start_fragment
                 start = start_offset
@@ -750,7 +744,7 @@ class LinearDrawer(AbstractDrawer):
                     feature_box, label = self.get_feature_sigil(feature, start,
                                                                 self.pagewidth,
                                                                 fragment)
-                    
+
                     fragment += 1   # move to next fragment
                     start = 0       # start next sigil from start of fragment
                     feature_boxes.append((feature_box, label))
@@ -772,7 +766,7 @@ class LinearDrawer(AbstractDrawer):
         startB = cross_link.startB
         endA = cross_link.endA
         endB = cross_link.endB
-           
+
         if not self.is_in_bounds(startA) \
         and not self.is_in_bounds(endA):
             return None
@@ -780,16 +774,21 @@ class LinearDrawer(AbstractDrawer):
         and not self.is_in_bounds(endB):
             return None
 
-        if startA < self.start: startA = self.start
-        if startB < self.start: startB = self.start
-        if self.end < endA: endA = self.end
-        if self.end < endB: endB = self.end
+        if startA < self.start:
+            startA = self.start
+        if startB < self.start:
+            startB = self.start
+        if self.end < endA:
+            endA = self.end
+        if self.end < endB:
+            endB = self.end
 
         trackobjA = cross_link._trackA(self._parent.tracks.values())
         trackobjB = cross_link._trackB(self._parent.tracks.values())
         assert trackobjA is not None
         assert trackobjB is not None
-        if trackobjA == trackobjB: raise NotImplementedError()
+        if trackobjA == trackobjB:
+            raise NotImplementedError()
 
         if trackobjA.start is not None:
             if endA < trackobjA.start:
@@ -814,19 +813,10 @@ class LinearDrawer(AbstractDrawer):
                 trackA = track_level
             if track == trackobjB:
                 trackB = track_level
-        if trackA == trackB: raise NotImplementedError()
+        if trackA == trackB:
+            raise NotImplementedError()
 
-        if cross_link.color == colors.white and border is None:   # Force black border on 
-            strokecolor = colors.black                 # white boxes with
-        elif cross_link.border is None:                           # undefined border, else
-            strokecolor = cross_link.color                        # use fill color
-        if cross_link.border:
-            if not isinstance(cross_link.border, colors.Color):
-                raise ValueError("Invalid border color %s" % repr(cross_link.border))
-            strokecolor = cross_link.border
-        else:
-            #e.g. False
-            strokecolor = None
+        strokecolor, fillcolor = _stroke_and_fill_colors(cross_link.color, cross_link.border)
 
         allowed_fragments = self.fragment_limits.keys()
 
@@ -851,12 +841,12 @@ class LinearDrawer(AbstractDrawer):
             btmA += self.fragment_lines[fragment][0]
             ctrA += self.fragment_lines[fragment][0]
             topA += self.fragment_lines[fragment][0]
-    
+
             btmB, ctrB, topB = self.track_offsets[trackB]
             btmB += self.fragment_lines[fragment][0]
             ctrB += self.fragment_lines[fragment][0]
             topB += self.fragment_lines[fragment][0]
-    
+
             if self.fragment_limits[fragment][1] < endA:
                 xAe = self.x0 + self.pagewidth
                 crop_rightA = True
@@ -915,9 +905,9 @@ class LinearDrawer(AbstractDrawer):
                                  self.x0 , 0.7*yA + 0.3*yB]
                 answer.append(Polygon([xAs, yA, xAe, yA] + extra,
                                strokeColor=strokecolor,
-                               fillColor=cross_link.color,
+                               fillColor=fillcolor,
                                #default is mitre/miter which can stick out too much:
-                               strokeLineJoin=1, #1=round
+                               strokeLineJoin=1,  # 1=round
                                strokewidth=0))
             elif fragment < start_fragmentA or end_fragmentA < fragment:
                 if cross_link.flip:
@@ -935,45 +925,45 @@ class LinearDrawer(AbstractDrawer):
                                  self.x0 , 0.3*yA + 0.7*yB]
                 answer.append(Polygon([xBs, yB, xBe, yB] + extra,
                                strokeColor=strokecolor,
-                               fillColor=cross_link.color,
+                               fillColor=fillcolor,
                                #default is mitre/miter which can stick out too much:
-                               strokeLineJoin=1, #1=round
+                               strokeLineJoin=1,  # 1=round
                                strokewidth=0))
-            elif cross_link.flip and ((crop_leftA and not crop_rightA) or \
+            elif cross_link.flip and ((crop_leftA and not crop_rightA) or
                                     (crop_leftB and not crop_rightB)):
                 #On left end of fragment... force "crossing" to margin
                 answer.append(Polygon([xAs, yA, xAe, yA,
                                        self.x0, 0.5 * (yA + yB),
                                        xBe, yB, xBs, yB],
                                strokeColor=strokecolor,
-                               fillColor=cross_link.color,
+                               fillColor=fillcolor,
                                #default is mitre/miter which can stick out too much:
-                               strokeLineJoin=1, #1=round
+                               strokeLineJoin=1,  # 1=round
                                strokewidth=0))
-            elif cross_link.flip and ((crop_rightA and not crop_leftA) or \
+            elif cross_link.flip and ((crop_rightA and not crop_leftA) or
                                       (crop_rightB and not crop_leftB)):
                 #On right end... force "crossing" to margin
                 answer.append(Polygon([xAs, yA, xAe, yA,
                                        xBe, yB, xBs, yB,
                                        self.x0 + self.pagewidth, 0.5 * (yA + yB)],
                                strokeColor=strokecolor,
-                               fillColor=cross_link.color,
+                               fillColor=fillcolor,
                                #default is mitre/miter which can stick out too much:
-                               strokeLineJoin=1, #1=round
+                               strokeLineJoin=1,  # 1=round
                                strokewidth=0))
             elif cross_link.flip:
                 answer.append(Polygon([xAs, yA, xAe, yA, xBs, yB, xBe, yB],
                                strokeColor=strokecolor,
-                               fillColor=cross_link.color,
+                               fillColor=fillcolor,
                                #default is mitre/miter which can stick out too much:
-                               strokeLineJoin=1, #1=round
+                               strokeLineJoin=1,  # 1=round
                                strokewidth=0))
             else:
                 answer.append(Polygon([xAs, yA, xAe, yA, xBe, yB, xBs, yB],
                                strokeColor=strokecolor,
-                               fillColor=cross_link.color,
+                               fillColor=fillcolor,
                                #default is mitre/miter which can stick out too much:
-                               strokeLineJoin=1, #1=round
+                               strokeLineJoin=1,  # 1=round
                                strokewidth=0))
         return answer
 
@@ -1010,11 +1000,13 @@ class LinearDrawer(AbstractDrawer):
             1/0
 
         # Distribution dictionary for various ways of drawing the feature
-        # Each method takes the corners of a containing box and a color
-        # as argument
-        draw_methods = {'BOX': draw_box,
-                        'ARROW': draw_arrow,
+        draw_methods = {'BOX': self._draw_sigil_box,
+                        'ARROW': self._draw_sigil_arrow,
+                        'BIGARROW': self._draw_sigil_big_arrow,
+                        'OCTO': self._draw_sigil_octo,
+                        'JAGGY': self._draw_sigil_jaggy,
                         }
+
         method = draw_methods[feature.sigil]
         kwargs['head_length_ratio'] = feature.arrowhead_length
         kwargs['shaft_height_ratio'] = feature.arrowshaft_height
@@ -1025,21 +1017,16 @@ class LinearDrawer(AbstractDrawer):
             kwargs["hrefURL"] = feature.url
             kwargs["hrefTitle"] = feature.name
 
-        strand = feature.strand
-        
-        # Get sigil for the feature, location dependent on the feature strand
-        if strand == 1:
-            sigil = method((x0, ctr), (x1, top), color=feature.color,
-                           border=feature.border,
-                           orientation='right', **kwargs)
-        elif strand == -1:
-            sigil = method((x1, btm), (x0, ctr), color=feature.color,
-                           border=feature.border,
-                           orientation='left', **kwargs)
+        # Get sigil for the feature, give it the bounding box straddling
+        # the axis (it decides strand specific placement)
+        sigil = method(btm, ctr, top, x0, x1, strand=feature.strand,
+                       color=feature.color, border=feature.border,
+                       **kwargs)
+
+        if feature.label_strand:
+            strand = feature.label_strand
         else:
-            sigil = method((x0, btm), (x1, top), color=feature.color,
-                           border=feature.border,
-                           **kwargs)
+            strand = feature.strand
         if feature.label:   # Feature requires a label
             label = String(0, 0, feature.name,
                            fontName=feature.label_font,
@@ -1048,31 +1035,32 @@ class LinearDrawer(AbstractDrawer):
             labelgroup = Group(label)
             # Feature is on top, or covers both strands (location affects
             # the height and rotation of the label)
-            if feature.strand in (0, 1):    
+            if strand != -1:
                 rotation = angle2trig(feature.label_angle)
-                if feature.label_position in ('start', "5'", 'left'):
-                    pos = x0
+                if feature.label_position in ('end', "3'", 'right'):
+                    pos = x1
                 elif feature.label_position in ('middle', 'center', 'centre'):
                     pos = (x1 + x0)/2.
                 else:
-                    pos = x1
+                    # Default to start, i.e. 'start', "5'", 'left'
+                    pos = x0
                 labelgroup.transform = (rotation[0], rotation[1], rotation[2],
                                         rotation[3], pos, top)
             else:   # Feature on bottom strand
-                rotation = angle2trig(feature.label_angle + 180)                        
-                if feature.label_position in ('start', "5'", 'left'):
-                    pos = x1
+                rotation = angle2trig(feature.label_angle + 180)
+                if feature.label_position in ('end', "3'", 'right'):
+                    pos = x0
                 elif feature.label_position in ('middle', 'center', 'centre'):
                     pos = (x1 + x0)/2.
                 else:
-                    pos = x0
+                    # Default to start, i.e. 'start', "5'", 'left'
+                    pos = x1
                 labelgroup.transform = (rotation[0], rotation[1], rotation[2],
                                         rotation[3], pos, btm)
         else:
             labelgroup = None
         return sigil, labelgroup
 
-                
     def draw_graph_set(self, set):
         """ draw_graph_set(self, set) -> ([element, element,...], [element, element,...])
 
@@ -1094,13 +1082,12 @@ class LinearDrawer(AbstractDrawer):
 
         return elements, []
 
-
     def draw_line_graph(self, graph):
         """ draw_line_graph(self, graph) -> [element, element,...]
 
             o graph     Graph object
 
-            Returns a line graph as a list of drawable elements 
+            Returns a line graph as a list of drawable elements
         """
         #print '\tdraw_line_graph'
         line_elements = []                  # Holds drawable elements
@@ -1113,14 +1100,14 @@ class LinearDrawer(AbstractDrawer):
         datarange = maxval - minval
         if datarange == 0:
             datarange = trackheight
-        
+
         start, end = self._current_track_start_end()
         data = graph[start:end]
 
         # midval is the value at which the x-axis is plotted, and is the
         # central ring in the track
         if graph.center is None:
-            midval = (maxval + minval)/2.    
+            midval = (maxval + minval)/2.
         else:
             midval = graph.center
         # Whichever is the greatest difference: max-midval or min-midval, is
@@ -1136,7 +1123,7 @@ class LinearDrawer(AbstractDrawer):
                 self.fragment_lines[lastfrag][0] + ctr
         lastval = val
         # Add a series of lines linking consecutive data points
-        for pos, val in data:   
+        for pos, val in data:
             frag, x = self.canvas_location(pos)
             x += self.x0        # next xy co-ords
             y = trackheight*(val-midval)/resolution + \
@@ -1158,9 +1145,8 @@ class LinearDrawer(AbstractDrawer):
                                           strokeColor = graph.poscolor,
                                           strokeWidth = graph.linewidth))
             lastfrag, lastx, lasty, lastval = frag, x, y, val
-            
-        return line_elements
 
+        return line_elements
 
     def draw_heat_graph(self, graph):
         """ draw_heat_graph(self, graph) -> [element, element,...]
@@ -1182,7 +1168,7 @@ class LinearDrawer(AbstractDrawer):
         midval = (maxval + minval)/2.    # mid is the value at the X-axis
         btm, ctr, top = self.track_offsets[self.current_track_level]
         trackheight = (top-btm)
-        
+
         start, end = self._current_track_start_end()
         data = intermediate_points(start, end, graph[start:end])
 
@@ -1198,13 +1184,13 @@ class LinearDrawer(AbstractDrawer):
             fragment1, x1 = self.canvas_location(pos1)
             x0, x1 = self.x0 + x0, self.x0 + x1     # account for margin
             #print 'x1 before:', x1
-            
+
             # Calculate the heat color, based on the differential between
             # the value and the median value
             heat = colors.linearlyInterpolatedColor(graph.poscolor,
                                                     graph.negcolor,
                                                     maxval, minval, val)
-            
+
             # Draw heat box
             if fragment0 == fragment1:  # Box is contiguous on one fragment
                 if pos1 >= self.fragment_limits[fragment0][1]:
@@ -1235,11 +1221,10 @@ class LinearDrawer(AbstractDrawer):
                 # Add the last part of the bar
                 #print 'x1 after:', x1, '\n'
                 heat_elements.append(draw_box((self.x0, tbtm), (x1, ttop),
-                                              color=heat, border=None))                    
+                                              color=heat, border=None))
 
         return heat_elements
 
-
     def draw_bar_graph(self, graph):
         """ draw_bar_graph(self, graph) -> [element, element,...]
 
@@ -1254,7 +1239,7 @@ class LinearDrawer(AbstractDrawer):
         # values go up in one color, negative go down in the alternative
         # color).
         bar_elements = []   # Holds drawable elements for the graph
-        
+
         # Set the number of pixels per unit for the data
         data_quartiles = graph.quartiles()
         minval, maxval = data_quartiles[0],data_quartiles[4]
@@ -1267,7 +1252,7 @@ class LinearDrawer(AbstractDrawer):
         # midval is the value at which the x-axis is plotted, and is the
         # central ring in the track
         if graph.center is None:
-            midval = (maxval + minval)/2.    
+            midval = (maxval + minval)/2.
         else:
             midval = graph.center
 
@@ -1286,7 +1271,7 @@ class LinearDrawer(AbstractDrawer):
         resolution = max((midval-minval), (maxval-midval))
         if resolution == 0:
             resolution = trackheight
-        
+
         # Create elements for the bar graph based on newdata
         for pos0, pos1, val in data:
             fragment0, x0 = self.canvas_location(pos0)
@@ -1303,13 +1288,13 @@ class LinearDrawer(AbstractDrawer):
                 if pos1 >= self.fragment_limits[fragment0][1]:
                     x1 = self.xlim
                 tctr = ctr + self.fragment_lines[fragment0][0]
-                barval += tctr                
+                barval += tctr
                 bar_elements.append(draw_box((x0, tctr), (x1, barval),
                                              color=barcolor))
-            else:   # Box is split over two or more fragments                       
+            else:   # Box is split over two or more fragments
                 fragment = fragment0
                 #if pos0 >= self.fragment_limits[fragment0][0]:
-                #    fragment += 1         
+                #    fragment += 1
                 start = x0
                 while self.fragment_limits[fragment][1] < pos1:
                     tctr = ctr + self.fragment_lines[fragment][0]
@@ -1327,7 +1312,6 @@ class LinearDrawer(AbstractDrawer):
 
         return bar_elements
 
-      
     def canvas_location(self, base):
         """ canvas_location(self, base) -> (int, float)
 
@@ -1350,4 +1334,99 @@ class LinearDrawer(AbstractDrawer):
         # Calculate number of pixels from start of fragment
         x_offset = 1. * self.pagewidth * base_offset / self.fragment_bases
         return fragment, x_offset
-        
+
+    def _draw_sigil_box(self, bottom, center, top, x1, x2, strand, **kwargs):
+        """Draw BOX sigil."""
+        if strand == 1:
+            y1 = center
+            y2 = top
+        elif strand == -1:
+            y1 = bottom
+            y2 = center
+        else:
+            y1 = bottom
+            y2 = top
+        return draw_box((x1,y1), (x2,y2), **kwargs)
+
+    def _draw_sigil_octo(self, bottom, center, top, x1, x2, strand, **kwargs):
+        """Draw OCTO sigil, a box with the corners cut off."""
+        if strand == 1:
+            y1 = center
+            y2 = top
+        elif strand == -1:
+            y1 = bottom
+            y2 = center
+        else:
+            y1 = bottom
+            y2 = top
+        return draw_cut_corner_box((x1,y1), (x2,y2), **kwargs)
+
+    def _draw_sigil_jaggy(self, bottom, center, top, x1, x2, strand,
+                          color, border=None, **kwargs):
+        """Draw JAGGY sigil.
+
+        Although we may in future expose the head/tail jaggy lengths, for now
+        both the left and right edges are drawn jagged.
+        """
+        if strand == 1:
+            y1 = center
+            y2 = top
+            teeth = 2
+        elif strand == -1:
+            y1 = bottom
+            y2 = center
+            teeth = 2
+        else:
+            y1 = bottom
+            y2 = top
+            teeth = 4
+
+        xmin = min(x1, x2)
+        xmax = max(x1, x2)
+        height = y2 - y1
+        boxwidth = x2 - x1
+        tooth_length = min(height/teeth, boxwidth*0.5)
+
+        headlength = tooth_length
+        taillength = tooth_length
+
+        strokecolor, color = _stroke_and_fill_colors(color, border)
+
+        points = []
+        for i in range(teeth):
+            points.extend((xmin, y1+i*height/teeth,
+                           xmin+taillength, y1+(i+1)*height/teeth))
+        for i in range(teeth):
+            points.extend((xmax, y1+(teeth-i)*height/teeth,
+                           xmax-headlength, y1+(teeth-i-1)*height/teeth))
+
+        return Polygon(points,
+                       strokeColor=strokecolor,
+                       strokeWidth=1,
+                       strokeLineJoin=1,  # 1=round
+                       fillColor=color,
+                       **kwargs)
+
+    def _draw_sigil_arrow(self, bottom, center, top, x1, x2, strand, **kwargs):
+        """Draw ARROW sigil."""
+        if strand == 1:
+            y1 = center
+            y2 = top
+            orientation = "right"
+        elif strand== -1:
+            y1 = bottom
+            y2 = center
+            orientation = "left"
+        else:
+            y1 = bottom
+            y2 = top
+            orientation = "right"  # backward compatibility
+        return draw_arrow((x1,y1), (x2,y2), orientation=orientation, **kwargs)
+
+    def _draw_sigil_big_arrow(self, bottom, center, top, x1, x2, strand, **kwargs):
+        """Draw BIGARROW sigil, like ARROW but straddles the axis."""
+        if strand == -1:
+            orientation = "left"
+        else:
+            orientation = "right"
+        return draw_arrow((x1,bottom), (x2,top), orientation=orientation, **kwargs)
