@@ -32,14 +32,14 @@ class Motif(object):
         self.name=""
 
     def _check_length(self, len):
-        if self.length==None:
+        if self.length is None:
             self.length = len
         elif self.length != len:
             print "len",self.length,self.instances, len
             raise ValueError("You can't change the length of the motif")
 
     def _check_alphabet(self,alphabet):
-        if self.alphabet==None:
+        if self.alphabet is None:
             self.alphabet=alphabet
         elif self.alphabet != alphabet:
                 raise ValueError("Wrong Alphabet")
@@ -106,7 +106,10 @@ class Motif(object):
                 #counting the occurences of letters in instances
                 for seq in self.instances:
                     #dict[seq[i]]=dict[seq[i]]+1
-                    dict[seq[i]]+=1
+                    try:
+                        dict[seq[i]]+=1
+                    except KeyError: #we need to ignore non-alphabet letters
+                        pass
             self._pwm.append(FreqTable.FreqTable(dict,FreqTable.COUNT,self.alphabet)) 
         self._pwm_is_current=1
         return self._pwm
@@ -325,7 +328,7 @@ class Motif(object):
         """
         calculates the dist_dpq measure with a given offset.
 
-        offset should satisfy 0<=offset<=self.length
+        offset should satisfy 0<=offset<=len(self)
         """
         def dpq (f1,f2,alpha):
             s=0
@@ -373,8 +376,10 @@ class Motif(object):
 
     def __len__(self):
         """return the length of a motif
+
+        Please use this method (i.e. invoke len(m)) instead of refering to the m.length directly.
         """
-        if self.length==None:
+        if self.length is None:
             return 0
         else:
             return self.length
@@ -396,7 +401,7 @@ class Motif(object):
             self.make_instances_from_counts()
         str = ""
         for i,inst in enumerate(self.instances):
-            str = str + "> instance %d\n"%i + inst.tostring() + "\n"
+            str = str + ">instance%d\n"%i + inst.tostring() + "\n"
             
         return str       
 
@@ -437,7 +442,7 @@ class Motif(object):
 
         self.counts = {}
         self.has_counts=True
-        if letters==None:
+        if letters is None:
             letters=self.alphabet.letters
         self.length=0
         for i in letters:
@@ -448,14 +453,14 @@ class Motif(object):
                 self.counts[k].append(v)
             self.length+=1
         self.set_mask("*"*self.length)
-        if make_instances==True:
+        if make_instances is True:
             self.make_instances_from_counts()
         return self
         
     def _from_horiz_matrix(self,stream,letters=None,make_instances=False):
         """reads a horizontal count matrix from stream and fill in the counts.
         """
-        if letters==None:
+        if letters is None:
             letters=self.alphabet.letters
         self.counts = {}
         self.has_counts=True
@@ -476,7 +481,7 @@ class Motif(object):
         l = len(self.counts[letters[0]])
         self.length=l
         self.set_mask("*"*l)
-        if make_instances==True:
+        if make_instances is True:
             self.make_instances_from_counts()
         return self
         
@@ -695,7 +700,7 @@ class Motif(object):
         """Return string representation of the motif as  a matrix.
         
         """
-        if letters==None:
+        if letters is None:
             letters=self.alphabet.letters
         self._pwm_is_current=False
         pwm=self.pwm(laplace=False)
@@ -709,7 +714,7 @@ class Motif(object):
         """Return string representation of the motif as  a matrix.
         
         """
-        if letters==None:
+        if letters is None:
             letters=self.alphabet.letters
         res=""
         if normalized: #output PWM
@@ -755,24 +760,49 @@ class Motif(object):
     def scanPWM(self,seq):
         """Matrix of log-odds scores for a nucleotide sequence.
  
-        scans (using a fast C extension) a nucleotide sequence and returns
-        the matrix of log-odds scores for all positions
+        scans a nucleotide sequence and returns the matrix of log-odds
+        scores for all positions.
 
-        - the result is a one-dimensional numpy array
+        - the result is a one-dimensional list or numpy array
         - the sequence can only be a DNA sequence
         - the search is performed only on one strand
         """
-        if self.alphabet!=IUPAC.unambiguous_dna:
-            raise ValueError("Wrong alphabet! Use only with DNA motifs")
-        if seq.alphabet!=IUPAC.unambiguous_dna:
-            raise ValueError("Wrong alphabet! Use only with DNA sequences")
+        #TODO - Code itself tolerates ambiguous bases (as NaN).
+        if not isinstance(self.alphabet, IUPAC.IUPACUnambiguousDNA):
+            raise ValueError("PSSM has wrong alphabet: %s - Use only with DNA motifs" \
+                                 % self.alphabet)
+        if not isinstance(seq.alphabet, IUPAC.IUPACUnambiguousDNA):
+            raise ValueError("Sequence has wrong alphabet: %r - Use only with DNA sequences" \
+                                 % sequence.alphabet)
 
-        import numpy
+        seq = seq.tostring()
+
+        # check if the fast C code can be used
+        try:
+            import _pwm
+        except ImportError:
+            # use the slower Python code otherwise
+            return self._pwm_calculate(seq)
+        
         # get the log-odds matrix into a proper shape
-        # (each column contains sorted (ACGT) log-odds values)
-        logodds=numpy.array([[y[1] for y in sorted(x.items())] \
-                             for x in self.log_odds()]).transpose()
-        
-        import _pwm
-        
-        return _pwm.calculate(seq.tostring(),logodds)
+        # (each row contains sorted (ACGT) log-odds values)
+        logodds=[[y[1] for y in sorted(x.items())] for x in self.log_odds()]
+        return _pwm.calculate(seq, logodds)
+
+    def _pwm_calculate(self, sequence):
+        logodds = self.log_odds()
+        m = len(logodds)
+        s = len(sequence)
+        n = s - m + 1
+        result = [None] * n
+        for i in xrange(n):
+            score = 0.0
+            for j in xrange(m):
+                c = sequence[i+j]
+                temp = logodds[j].get(c)
+                if temp is None:
+                    break
+                score += temp
+            else:
+                result[i] = score
+        return result
