@@ -2,20 +2,35 @@
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
-"""
-Module containing different tools for sequence motif analysis.
+"""Tools for sequence motif analysis (DEPRECATED, see Bio.motifs instead).
 
-it contains the core Motif class containing various I/O methods
-as well as methods for motif comparisons and motif searching in sequences.
-It also inlcudes functionality for parsing AlignACE and MEME programs
-"""
-from _Motif import Motif
-import Parsers.AlignAce
-import Parsers.MEME
-from Thresholds import ScoreDistribution
+This module (Bio.Motif) has been deprecated and will be removed in a
+future release of release of Biopython. Please use the new module
+Bio.motifs instead.
 
-_parsers={"AlignAce":Parsers.AlignAce.read,
-          "MEME":Parsers.MEME.read
+This contains the core Motif class containing various I/O methods as
+well as methods for motif comparisons and motif searching in sequences.
+It also inlcudes functionality for parsing AlignACE and MEME programs.
+"""
+
+import warnings
+from Bio import BiopythonDeprecationWarning
+warnings.warn("The module Bio.Motif has been deprecated and will be "
+              "removed in a future release of Biopython. Instead "
+              "please use the new module Bio.motifs instead. Please "
+              "be aware that though the functionality of Bio.Motif "
+              "is retained (and extended) in Bio.motifs, usage may "
+              "be different.",
+              BiopythonDeprecationWarning)
+
+
+from Bio.Motif._Motif import Motif
+from Bio.Motif.Parsers.AlignAce import read as _AlignAce_read
+from Bio.Motif.Parsers.MEME import read as _MEME_read
+from Bio.Motif.Thresholds import ScoreDistribution
+
+_parsers={"AlignAce" : _AlignAce_read,
+          "MEME" : _MEME_read,
           }
 
 def _from_pfm(handle):
