@@ -219,8 +219,8 @@ def draw_ascii(tree, file=sys.stdout, column_width=80):
             for subclade in clade:
                 if subclade not in positions:
                     calc_row(subclade)
-            positions[clade] = (positions[clade.clades[0]] +
-                                positions[clade.clades[-1]]) / 2
+            positions[clade] = ((positions[clade.clades[0]] +
+                                 positions[clade.clades[-1]]) // 2)
 
         calc_row(tree.root)
         return positions
@@ -255,21 +255,31 @@ def draw_ascii(tree, file=sys.stdout, column_width=80):
         line = ''.join(row).rstrip()
         # Add labels for terminal taxa in the right margin
         if idx % 2 == 0:
-            line += ' ' + str(taxa[idx/2])
+            line += ' ' + str(taxa[idx//2])
         file.write(line + '\n')
     file.write('\n')
 
 
 def draw(tree, label_func=str, do_show=True, show_confidence=True,
         # For power users
-        axes=None, branch_labels=None):
+        axes=None, branch_labels=None, *args, **kwargs):
     """Plot the given tree using matplotlib (or pylab).
 
     The graphic is a rooted tree, drawn with roughly the same algorithm as
     draw_ascii.
 
-    Visual aspects of the plot can be modified using pyplot's own functions and
-    objects (via pylab or matplotlib). In particular, the pyplot.rcParams
+    Additional keyword arguments passed into this function are used as pyplot
+    options. The input format should be in the form of:
+    pyplot_option_name=(tuple), pyplot_option_name=(tuple, dict), or
+    pyplot_option_name=(dict).
+
+    Example using the pyplot options 'axhspan' and 'axvline':
+
+    >>> Phylo.draw(tree, axhspan=((0.25, 7.75), {'facecolor':'0.5'}),
+    ...     axvline={'x':'0', 'ymin':'0', 'ymax':'1'})
+
+    Visual aspects of the plot can also be modified using pyplot's own functions
+    and objects (via pylab or matplotlib). In particular, the pyplot.rcParams
     object can be used to scale the font size (rcParams["font.size"]) and line
     width (rcParams["lines.linewidth"]).
 
@@ -295,6 +305,7 @@ def draw(tree, label_func=str, do_show=True, show_confidence=True,
             or label the branches with something other than confidence, then use
             this option.
     """
+
     try:
         import matplotlib.pyplot as plt
     except ImportError:
@@ -305,6 +316,12 @@ def draw(tree, label_func=str, do_show=True, show_confidence=True,
             raise MissingPythonDependencyError(
                     "Install matplotlib or pylab if you want to use draw.")
 
+    import matplotlib.collections as mpcollections
+
+    # Arrays that store lines for the plot of clades
+    horizontal_linecollections = []
+    vertical_linecollections = []
+
     # Options for displaying branch labels / confidence
     def conf2str(conf):
         if int(conf) == conf:
@@ -377,6 +394,25 @@ def draw(tree, label_func=str, do_show=True, show_confidence=True,
     elif not isinstance(axes, plt.matplotlib.axes.Axes):
         raise ValueError("Invalid argument for axes: %s" % axes)
 
+    def draw_clade_lines(use_linecollection=False, orientation='horizontal',
+                         y_here=0, x_start=0, x_here=0, y_bot=0, y_top=0,
+                         color='black', lw='.1'):
+        """Create a line with or without a line collection object.
+
+        Graphical formatting of the lines representing clades in the plot can be
+        customized by altering this function.
+        """
+        if (use_linecollection==False and orientation=='horizontal'):
+            axes.hlines(y_here, x_start, x_here, color=color, lw=lw)
+        elif (use_linecollection==True and orientation=='horizontal'):
+            horizontal_linecollections.append(mpcollections.LineCollection(
+            [[(x_start,y_here), (x_here,y_here)]], color=color, lw=lw),)
+        elif (use_linecollection==False and orientation=='vertical'):
+            axes.vlines(x_here, y_bot, y_top, color=color)
+        elif (use_linecollection==True and orientation=='vertical'):
+            vertical_linecollections.append(mpcollections.LineCollection(
+            [[(x_here,y_bot), (x_here,y_top)]], color=color, lw=lw),)
+
     def draw_clade(clade, x_start, color, lw):
         """Recursively draw a tree, down from the given clade."""
         x_here = x_posns[clade]
@@ -387,7 +423,8 @@ def draw(tree, label_func=str, do_show=True, show_confidence=True,
         if hasattr(clade, 'width') and clade.width is not None:
             lw = clade.width * plt.rcParams['lines.linewidth']
         # Draw a horizontal line from start to here
-        axes.hlines(y_here, x_start, x_here, color=color, lw=lw)
+        draw_clade_lines(use_linecollection=True, orientation='horizontal',
+            y_here=y_here, x_start=x_start, x_here=x_here, color='black', lw=lw)
         # Add node/taxon labels
         label = label_func(clade)
         if label not in (None, clade.__class__.__name__):
@@ -402,13 +439,21 @@ def draw(tree, label_func=str, do_show=True, show_confidence=True,
             y_top = y_posns[clade.clades[0]]
             y_bot = y_posns[clade.clades[-1]]
             # Only apply widths to horizontal lines, like Archaeopteryx
-            axes.vlines(x_here, y_bot, y_top, color=color)
+            draw_clade_lines(use_linecollection=True, orientation='vertical',
+                x_here=x_here, y_bot=y_bot, y_top=y_top, color='black', lw=lw)
             # Draw descendents
             for child in clade:
                 draw_clade(child, x_here, color, lw)
 
     draw_clade(tree.root, 0, 'k', plt.rcParams['lines.linewidth'])
 
+    # If line collections were used to create clade lines, here they are added
+    # to the pyplot plot.
+    for i in horizontal_linecollections:
+        axes.add_collection(i)
+    for i in vertical_linecollections:
+        axes.add_collection(i)
+
     # Aesthetics
 
     if hasattr(tree, 'name') and tree.name:
@@ -421,5 +466,23 @@ def draw(tree, label_func=str, do_show=True, show_confidence=True,
     # Also invert the y-axis (origin at the top)
     # Add a small vertical margin, but avoid including 0 and N+1 on the y axis
     axes.set_ylim(max(y_posns.itervalues()) + 0.8, 0.2)
+
+    # Parse and process key word arguments as pyplot options
+    for key, value in kwargs.iteritems():
+        try:
+            # Check that the pyplot option input is iterable, as required
+            [i for i in value]
+        except TypeError:
+            raise ValueError('Keyword argument "%s=%s" is not in the format '
+                'pyplot_option_name=(tuple), pyplot_option_name=(tuple, dict),'
+                ' or pyplot_option_name=(dict) '
+                % (key, value))
+        if isinstance(value, dict):
+            getattr(plt, str(key))(**dict(value))
+        elif not (isinstance(value[0], tuple)):
+            getattr(plt, str(key))(*value)
+        elif (isinstance(value[0], tuple)):
+            getattr(plt, str(key))(*value[0], **dict(value[1]))
+
     if do_show:
         plt.show()
