@@ -11,7 +11,7 @@
 Bio.SeqIO is also documented at U{http://biopython.org/wiki/SeqIO} and by
 a whole chapter in our tutorial:
  - U{http://biopython.org/DIST/docs/tutorial/Tutorial.html}
- - U{http://biopython.org/DIST/docs/tutorial/Tutorial.pdf}  
+ - U{http://biopython.org/DIST/docs/tutorial/Tutorial.pdf}
 
 Input
 =====
@@ -112,7 +112,7 @@ keep the output 100% identical to the input). For example,
     >>> record_dict = SeqIO.index("Fasta/f002", "fasta")
     >>> len(record_dict)
     3
-    >>> print record_dict.get_raw("gi|1348917|gb|G26685|G26685")
+    >>> print record_dict.get_raw("gi|1348917|gb|G26685|G26685").decode()
     >gi|1348917|gb|G26685|G26685 human STS STS_D11734.
     CGGAGCCAGCGAGCATATGCTGCATGAGGACCTTTCTATCTTACATTATGGCTGGGAATCTTACTCTTTC
     ATCTGATACCTTGTTCAGATTTCAAAATAGTTGTAGCCTTATCCTGGTTTTACAGATGTGAAACTTTCAA
@@ -133,7 +133,10 @@ keep the output 100% identical to the input). For example,
     <BLANKLINE>
 
 Here the original file and what Biopython would output differ in the line
-wrapping.
+wrapping. Also note that under Python 3, the get_raw method will return a
+bytes string, hence the use of decode to turn it into a (unicode) string.
+This is uncessary on Python 2.
+
 
 Input - Alignments
 ==================
@@ -172,6 +175,7 @@ Or, using a handle::
 You are expected to call this function once (with all your records) and if
 using a handle, make sure you close it to flush the data to the hard disk.
 
+
 Output - Advanced
 =================
 The effect of calling write() multiple times on a single file will vary
@@ -201,11 +205,13 @@ the Bio.SeqIO.write(...) function for sequence file conversion. Using
 generator expressions or generator functions provides a memory efficient way
 to perform filtering or other extra operations as part of the process.
 
+
 File Formats
 ============
 When specifying the file format, use lowercase strings.  The same format
 names are also used in Bio.AlignIO and include the following:
 
+ - abif    - Applied Biosystem's sequencing trace format
  - ace     - Reads the contig sequences from an ACE assembly file.
  - embl    - The EMBL flat file format. Uses Bio.GenBank internally.
  - fasta   - The generic sequence file format where each record starts with
@@ -217,16 +223,21 @@ names are also used in Bio.AlignIO and include the following:
  - fastq-solexa - Original Solexa/Illumnia variant of the FASTQ format which
              encodes Solexa quality scores (not PHRED quality scores) with an
              ASCII offset of 64.
- - fastq-illumina - Solexa/Illumnia 1.3+ variant of the FASTQ format which
-             encodes PHRED quality scores with an ASCII offset of 64 (not 33).
+ - fastq-illumina - Solexa/Illumina 1.3 to 1.7 variant of the FASTQ format
+             which encodes PHRED quality scores with an ASCII offset of 64
+             (not 33). Note as of version 1.8 of the CASAVA pipeline Illumina
+             will produce FASTQ files using the standard Sanger encoding.
  - genbank - The GenBank or GenPept flat file format.
  - gb      - An alias for "genbank", for consistency with NCBI Entrez Utilities
  - ig      - The IntelliGenetics file format, apparently the same as the
              MASE alignment format.
+ - imgt    - An EMBL like format from IMGT where the feature tables are more
+             indented to allow for longer feature types.
  - phd     - Output from PHRED, used by PHRAP and CONSED for input.
  - pir     - A "FASTA like" format introduced by the National Biomedical
              Research Foundation (NBRF) for the Protein Information Resource
              (PIR) database, now part of UniProt.
+ - seqxml  - SeqXML, simple XML format described in Schmitt et al (2011).
  - sff     - Standard Flowgram Format (SFF), typical output from Roche 454.
  - sff-trim - Standard Flowgram Format (SFF) with given trimming applied.
  - swiss   - Plain text Swiss-Prot aka UniProt format.
@@ -237,6 +248,8 @@ names are also used in Bio.AlignIO and include the following:
  - qual    - A "FASTA like" format holding PHRED quality values from
              sequencing DNA, but no actual sequences (usually provided
              in separate FASTA files).
+ - uniprot-xml - The UniProt XML format (replacement for the SwissProt plain
+             text format which we call "swiss")
 
 Note that while Bio.SeqIO can read all the above file formats, it cannot
 write to all of them.
@@ -245,7 +258,11 @@ You can also use any file format supported by Bio.AlignIO, such as "nexus",
 "phlip" and "stockholm", which gives you access to the individual sequences
 making up each alignment as SeqRecords.
 """
-__docformat__ = "epytext en" #not just plaintext
+
+# For using with statement in Python 2.5 or Jython
+from __future__ import with_statement
+
+__docformat__ = "epytext en"  # not just plaintext
 
 #TODO
 # - define policy on reading aligned sequences with gaps in
@@ -256,7 +273,7 @@ __docformat__ = "epytext en" #not just plaintext
 #   parser would fail.
 #
 # - MSF multiple alignment format, aka GCG, aka PileUp format (*.msf)
-#   http://www.bioperl.org/wiki/MSF_multiple_alignment_format 
+#   http://www.bioperl.org/wiki/MSF_multiple_alignment_format
 
 """
 FAO BioPython Developers
@@ -291,22 +308,26 @@ See also http://biopython.org/wiki/SeqIO_dev
 --Peter
 """
 
-from Bio.Seq import Seq
+
+from Bio.File import as_handle
 from Bio.SeqRecord import SeqRecord
 from Bio.Align import MultipleSeqAlignment
-from Bio.Align.Generic import Alignment
 from Bio.Alphabet import Alphabet, AlphabetEncoder, _get_base_alphabet
 
+import AbiIO
 import AceIO
 import FastaIO
-import IgIO #IntelliGenetics or MASE format
-import InsdcIO #EMBL and GenBank
+import IgIO  # IntelliGenetics or MASE format
+import InsdcIO  # EMBL and GenBank
+import PdbIO
 import PhdIO
 import PirIO
+import SeqXmlIO
 import SffIO
 import SwissIO
 import TabIO
-import QualityIO #FastQ and qual files
+import QualityIO  # FastQ and qual files
+import UniprotIO
 
 
 #Convention for format names is "mainname-subtype" in lower case.
@@ -318,43 +339,53 @@ import QualityIO #FastQ and qual files
 #
 #Most alignment file formats will be handled via Bio.AlignIO
 
-_FormatToIterator = {"fasta" : FastaIO.FastaIterator,
-                     "gb" : InsdcIO.GenBankIterator,
-                     "genbank" : InsdcIO.GenBankIterator,
-                     "genbank-cds" : InsdcIO.GenBankCdsFeatureIterator,
-                     "embl" : InsdcIO.EmblIterator,
-                     "embl-cds" : InsdcIO.EmblCdsFeatureIterator,
-                     "ig" : IgIO.IgIterator,
-                     "swiss" : SwissIO.SwissIterator,
-                     "phd" : PhdIO.PhdIterator,
-                     "ace" : AceIO.AceIterator,
-                     "tab" : TabIO.TabIterator,
-                     "pir" : PirIO.PirIterator,
-                     "fastq" : QualityIO.FastqPhredIterator,
-                     "fastq-sanger" : QualityIO.FastqPhredIterator,
-                     "fastq-solexa" : QualityIO.FastqSolexaIterator,
-                     "fastq-illumina" : QualityIO.FastqIlluminaIterator,
-                     "qual" : QualityIO.QualPhredIterator,
+_FormatToIterator = {"fasta": FastaIO.FastaIterator,
+                     "gb": InsdcIO.GenBankIterator,
+                     "genbank": InsdcIO.GenBankIterator,
+                     "genbank-cds": InsdcIO.GenBankCdsFeatureIterator,
+                     "embl": InsdcIO.EmblIterator,
+                     "embl-cds": InsdcIO.EmblCdsFeatureIterator,
+                     "imgt": InsdcIO.ImgtIterator,
+                     "ig": IgIO.IgIterator,
+                     "swiss": SwissIO.SwissIterator,
+                     "pdb-atom": PdbIO.PdbAtomIterator,
+                     "pdb-seqres": PdbIO.PdbSeqresIterator,
+                     "phd": PhdIO.PhdIterator,
+                     "ace": AceIO.AceIterator,
+                     "tab": TabIO.TabIterator,
+                     "pir": PirIO.PirIterator,
+                     "fastq": QualityIO.FastqPhredIterator,
+                     "fastq-sanger": QualityIO.FastqPhredIterator,
+                     "fastq-solexa": QualityIO.FastqSolexaIterator,
+                     "fastq-illumina": QualityIO.FastqIlluminaIterator,
+                     "qual": QualityIO.QualPhredIterator,
                      "sff": SffIO.SffIterator,
                      #Not sure about this in the long run:
                      "sff-trim": SffIO._SffTrimIterator,
+                     "uniprot-xml": UniprotIO.UniprotIterator,
+                     "seqxml": SeqXmlIO.SeqXmlIterator,
+                     "abi": AbiIO.AbiIterator,
+                     "abi-trim": AbiIO._AbiTrimIterator,
                      }
 
-_FormatToWriter = {"fasta" : FastaIO.FastaWriter,
-                   "gb" : InsdcIO.GenBankWriter,
-                   "genbank" : InsdcIO.GenBankWriter,
-                   "embl" : InsdcIO.EmblWriter,
-                   "tab" : TabIO.TabWriter,
-                   "fastq" : QualityIO.FastqPhredWriter,
-                   "fastq-sanger" : QualityIO.FastqPhredWriter,
-                   "fastq-solexa" : QualityIO.FastqSolexaWriter,
-                   "fastq-illumina" : QualityIO.FastqIlluminaWriter,
-                   "phd" : PhdIO.PhdWriter,
-                   "qual" : QualityIO.QualPhredWriter,
-                   "sff" : SffIO.SffWriter,
+_FormatToWriter = {"fasta": FastaIO.FastaWriter,
+                   "gb": InsdcIO.GenBankWriter,
+                   "genbank": InsdcIO.GenBankWriter,
+                   "embl": InsdcIO.EmblWriter,
+                   "imgt": InsdcIO.ImgtWriter,
+                   "tab": TabIO.TabWriter,
+                   "fastq": QualityIO.FastqPhredWriter,
+                   "fastq-sanger": QualityIO.FastqPhredWriter,
+                   "fastq-solexa": QualityIO.FastqSolexaWriter,
+                   "fastq-illumina": QualityIO.FastqIlluminaWriter,
+                   "phd": PhdIO.PhdWriter,
+                   "qual": QualityIO.QualPhredWriter,
+                   "sff": SffIO.SffWriter,
+                   "seqxml": SeqXmlIO.SeqXmlWriter,
                    }
 
-_BinaryFormats = ["sff", "sff-trim"]
+_BinaryFormats = ["sff", "sff-trim", "abi", "abi-trim"]
+
 
 def write(sequences, handle, format):
     """Write complete set of sequences to a file.
@@ -383,48 +414,44 @@ def write(sequences, handle, format):
         #This raised an exception in order version of Biopython
         sequences = [sequences]
 
-    if isinstance(handle, basestring):
-        if format in _BinaryFormats :
-            handle = open(handle, "wb")
-        else :
-            handle = open(handle, "w")
-        handle_close = True
+    if format in _BinaryFormats:
+        mode = 'wb'
     else:
-        handle_close = False
+        mode = 'w'
 
-    #Map the file format to a writer class
-    if format in _FormatToWriter:
-        writer_class = _FormatToWriter[format]
-        count = writer_class(handle).write_file(sequences)
-    elif format in AlignIO._FormatToWriter:
-        #Try and turn all the records into a single alignment,
-        #and write that using Bio.AlignIO
-        alignment = MultipleSeqAlignment(sequences)
-        alignment_count = AlignIO.write([alignment], handle, format)
-        assert alignment_count == 1, "Internal error - the underlying writer " \
-           + " should have returned 1, not %s" % repr(alignment_count)
-        count = len(alignment)
-        del alignment_count, alignment
-    elif format in _FormatToIterator or format in AlignIO._FormatToIterator:
-        raise ValueError("Reading format '%s' is supported, but not writing" \
-                         % format)
-    else:
-        raise ValueError("Unknown format '%s'" % format)
+    with as_handle(handle, mode) as fp:
+        #Map the file format to a writer class
+        if format in _FormatToWriter:
+            writer_class = _FormatToWriter[format]
+            count = writer_class(fp).write_file(sequences)
+        elif format in AlignIO._FormatToWriter:
+            #Try and turn all the records into a single alignment,
+            #and write that using Bio.AlignIO
+            alignment = MultipleSeqAlignment(sequences)
+            alignment_count = AlignIO.write([alignment], fp, format)
+            assert alignment_count == 1, \
+                "Internal error - the underlying writer " \
+                " should have returned 1, not %s" % repr(alignment_count)
+            count = len(alignment)
+            del alignment_count, alignment
+        elif format in _FormatToIterator or format in AlignIO._FormatToIterator:
+            raise ValueError("Reading format '%s' is supported, but not writing"
+                             % format)
+        else:
+            raise ValueError("Unknown format '%s'" % format)
 
-    assert isinstance(count, int), "Internal error - the underlying %s " \
-           "writer should have returned the record count, not %s" \
-           % (format, repr(count))
+        assert isinstance(count, int), "Internal error - the underlying %s " \
+            "writer should have returned the record count, not %s" \
+            % (format, repr(count))
 
-    if handle_close:
-        handle.close()
-    
     return count
-    
+
+
 def parse(handle, format, alphabet=None):
     r"""Turns a sequence file into an iterator returning SeqRecords.
 
      - handle   - handle to the file, or the filename as a string
-                  (note older verions of Biopython only took a handle).
+                  (note older versions of Biopython only took a handle).
      - format   - lower case string describing the file format.
      - alphabet - optional Alphabet object, useful when the sequence type
                   cannot be automatically inferred from the file itself
@@ -475,13 +502,11 @@ def parse(handle, format, alphabet=None):
     #string mode (see the leading r before the opening quote).
     from Bio import AlignIO
 
-    if isinstance(handle, basestring):
-        #Hack for SFF, will need to make this more general in future
-        if format in _BinaryFormats :
-            handle = open(handle, "rb")
-        else :
-            handle = open(handle, "rU")
-        #TODO - On Python 2.5+ use with statement to close handle
+    #Hack for SFF, will need to make this more general in future
+    if format in _BinaryFormats:
+        mode = 'rb'
+    else:
+        mode = 'rU'
 
     #Try and give helpful error messages:
     if not isinstance(format, basestring):
@@ -490,34 +515,32 @@ def parse(handle, format, alphabet=None):
         raise ValueError("Format required (lower case string)")
     if format != format.lower():
         raise ValueError("Format string '%s' should be lower case" % format)
-    if alphabet is not None and not (isinstance(alphabet, Alphabet) or \
+    if alphabet is not None and not (isinstance(alphabet, Alphabet) or
                                      isinstance(alphabet, AlphabetEncoder)):
         raise ValueError("Invalid alphabet, %s" % repr(alphabet))
 
-    #Map the file format to a sequence iterator:    
-    if format in _FormatToIterator:
-        iterator_generator = _FormatToIterator[format]
-        if alphabet is None:
-            return iterator_generator(handle)
-        try:
-            return iterator_generator(handle, alphabet=alphabet)
-        except:
-            return _force_alphabet(iterator_generator(handle), alphabet)
-    elif format in AlignIO._FormatToIterator:
-        #Use Bio.AlignIO to read in the alignments
-        #TODO - Can this helper function can be replaced with a generator
-        #expression, or something from itertools?
-        return _iterate_via_AlignIO(handle, format, alphabet)
-    else:
-        raise ValueError("Unknown format '%s'" % format)
+    with as_handle(handle, mode) as fp:
+        #Map the file format to a sequence iterator:
+        if format in _FormatToIterator:
+            iterator_generator = _FormatToIterator[format]
+            if alphabet is None:
+                i = iterator_generator(fp)
+            else:
+                try:
+                    i = iterator_generator(fp, alphabet=alphabet)
+                except TypeError:
+                    i = _force_alphabet(iterator_generator(fp), alphabet)
+        elif format in AlignIO._FormatToIterator:
+            #Use Bio.AlignIO to read in the alignments
+            i = (r for alignment in AlignIO.parse(fp, format,
+                                                  alphabet=alphabet)
+                 for r in alignment)
+        else:
+            raise ValueError("Unknown format '%s'" % format)
+        #This imposes some overhead... wait until we drop Python 2.4 to fix it
+        for r in i:
+            yield r
 
-#This is a generator function
-def _iterate_via_AlignIO(handle, format, alphabet):
-    """Iterate over all records in several alignments (PRIVATE)."""
-    from Bio import AlignIO
-    for align in AlignIO.parse(handle, format, alphabet=alphabet):
-        for record in align:
-            yield record
 
 def _force_alphabet(record_iterator, alphabet):
     """Iterate over records, over-riding the alphabet (PRIVATE)."""
@@ -529,15 +552,16 @@ def _force_alphabet(record_iterator, alphabet):
             record.seq.alphabet = alphabet
             yield record
         else:
-            raise ValueError("Specified alphabet %s clashes with "\
-                             "that determined from the file, %s" \
+            raise ValueError("Specified alphabet %s clashes with "
+                             "that determined from the file, %s"
                              % (repr(alphabet), repr(record.seq.alphabet)))
 
+
 def read(handle, format, alphabet=None):
     """Turns a sequence file into a single SeqRecord.
 
      - handle   - handle to the file, or the filename as a string
-                  (note older verions of Biopython only took a handle).
+                  (note older versions of Biopython only took a handle).
      - format   - string describing the file format.
      - alphabet - optional Alphabet object, useful when the sequence type
                   cannot be automatically inferred from the file itself
@@ -591,6 +615,7 @@ def read(handle, format, alphabet=None):
         raise ValueError("More than one record found in handle")
     return first
 
+
 def to_dict(sequences, key_function=None):
     """Turns a sequence iterator or list into a dictionary.
 
@@ -602,9 +627,9 @@ def to_dict(sequences, key_function=None):
     e.g. key_function = lambda rec : rec.name
     or,  key_function = lambda rec : rec.description.split()[0]
 
-    If key_function is ommitted then record.id is used, on the assumption
+    If key_function is omitted then record.id is used, on the assumption
     that the records objects returned are SeqRecords with a unique id.
-    
+
     If there are duplicate keys, an error is raised.
 
     Example usage, defaulting to using the record.id as key:
@@ -639,9 +664,9 @@ def to_dict(sequences, key_function=None):
     This approach is not suitable for very large sets of sequences, as all
     the SeqRecord objects are held in memory. Instead, consider using the
     Bio.SeqIO.index() function (if it supports your particular file format).
-    """    
+    """
     if key_function is None:
-        key_function = lambda rec : rec.id
+        key_function = lambda rec: rec.id
 
     d = dict()
     for record in sequences:
@@ -651,6 +676,7 @@ def to_dict(sequences, key_function=None):
         d[key] = record
     return d
 
+
 def index(filename, format, alphabet=None, key_function=None):
     """Indexes a sequence file and returns a dictionary like object.
 
@@ -662,7 +688,7 @@ def index(filename, format, alphabet=None, key_function=None):
      - key_function - Optional callback function which when given a
                   SeqRecord identifier string should return a unique
                   key for the dictionary.
-    
+
     This indexing function will return a dictionary like object, giving the
     SeqRecord objects as values:
 
@@ -681,7 +707,17 @@ def index(filename, format, alphabet=None, key_function=None):
     >>> print records.get("Missing", None)
     None
 
-    Note that this psuedo dictionary will not support all the methods of a
+    If the file is BGZF compressed, this is detected automatically. Ordinary
+    GZIP files are not supported:
+
+    >>> from Bio import SeqIO
+    >>> records = SeqIO.index("Quality/example.fastq.bgz", "fastq")
+    >>> len(records)
+    3
+    >>> print records["EAS54_6_R1_2_1_540_792"].seq
+    TTGGCAGGCCAAGGCCGATGGATCA
+
+    Note that this pseudo dictionary will not support all the methods of a
     true Python dictionary, for example values() is not defined since this
     would require loading all of the records into memory at once.
 
@@ -711,7 +747,7 @@ def index(filename, format, alphabet=None, key_function=None):
 
     As with the to_dict() function, by default the id string of each record
     is used as the key. You can specify a callback function to transform
-    this (the record identifier string) into your prefered key. For example:
+    this (the record identifier string) into your preferred key. For example:
 
     >>> from Bio import SeqIO
     >>> def make_tuple(identifier):
@@ -743,6 +779,8 @@ def index(filename, format, alphabet=None, key_function=None):
     would impose a severe performance penalty as it would require the file
     to be completely parsed while building the index. Right now this is
     usually avoided.
+
+    See also: Bio.SeqIO.index_db() and Bio.SeqIO.to_dict()
     """
     #Try and give helpful error messages:
     if not isinstance(filename, basestring):
@@ -753,41 +791,105 @@ def index(filename, format, alphabet=None, key_function=None):
         raise ValueError("Format required (lower case string)")
     if format != format.lower():
         raise ValueError("Format string '%s' should be lower case" % format)
-    if alphabet is not None and not (isinstance(alphabet, Alphabet) or \
+    if alphabet is not None and not (isinstance(alphabet, Alphabet) or
                                      isinstance(alphabet, AlphabetEncoder)):
         raise ValueError("Invalid alphabet, %s" % repr(alphabet))
 
-    #Map the file format to a sequence iterator:    
-    import _index #Lazy import
+    #Map the file format to a sequence iterator:
+    from _index import _FormatToRandomAccess # Lazy import
+    from Bio.File import _IndexedSeqFileDict
     try:
-        indexer = _index._FormatToIndexedDict[format]
+        proxy_class = _FormatToRandomAccess[format]
     except KeyError:
-        raise ValueError("Unsupported format '%s'" % format)
-    return indexer(filename, format, alphabet, key_function)
+        raise ValueError("Unsupported format %r" % format)
+    repr = "SeqIO.index(%r, %r, alphabet=%r, key_function=%r)" \
+        % (filename, format, alphabet, key_function)
+    return _IndexedSeqFileDict(proxy_class(filename, format, alphabet),
+                               key_function, repr, "SeqRecord")
 
-def to_alignment(sequences, alphabet=None, strict=True):
-    """Returns a multiple sequence alignment (DEPRECATED).
 
-     - sequences -An iterator that returns SeqRecord objects,
-                  or simply a list of SeqRecord objects.  All
-                  the record sequences must be the same length.
-     - alphabet - Optional alphabet.  Stongly recommended.
-     - strict   - Dummy argument, used to enable strict error
-                  checking of sequence lengths and alphabets.
-                  This is now always done.
+def index_db(index_filename, filenames=None, format=None, alphabet=None,
+             key_function=None):
+    """Index several sequence files and return a dictionary like object.
 
-    Using this function is now discouraged. You are now encouraged to use
-    Bio.AlignIO instead, e.g.
+    The index is stored in an SQLite database rather than in memory (as in the
+    Bio.SeqIO.index(...) function).
 
-    >>> from Bio import AlignIO
-    >>> filename = "Clustalw/protein.aln"
-    >>> alignment = AlignIO.read(filename, "clustal")
+     - index_filename - Where to store the SQLite index
+     - filenames - list of strings specifying file(s) to be indexed, or when
+                  indexing a single file this can be given as a string.
+                  (optional if reloading an existing index, but must match)
+     - format   - lower case string describing the file format
+                  (optional if reloading an existing index, but must match)
+     - alphabet - optional Alphabet object, useful when the sequence type
+                  cannot be automatically inferred from the file itself
+                  (e.g. format="fasta" or "tab")
+     - key_function - Optional callback function which when given a
+                  SeqRecord identifier string should return a unique
+                  key for the dictionary.
+
+    This indexing function will return a dictionary like object, giving the
+    SeqRecord objects as values:
+
+    >>> from Bio.Alphabet import generic_protein
+    >>> from Bio import SeqIO
+    >>> files = ["GenBank/NC_000932.faa", "GenBank/NC_005816.faa"]
+    >>> def get_gi(name):
+    ...     parts = name.split("|")
+    ...     i = parts.index("gi")
+    ...     assert i != -1
+    ...     return parts[i+1]
+    >>> idx_name = ":memory:" #use an in memory SQLite DB for this test
+    >>> records = SeqIO.index_db(idx_name, files, "fasta", generic_protein, get_gi)
+    >>> len(records)
+    95
+    >>> records["7525076"].description
+    'gi|7525076|ref|NP_051101.1| Ycf2 [Arabidopsis thaliana]'
+    >>> records["45478717"].description
+    'gi|45478717|ref|NP_995572.1| pesticin [Yersinia pestis biovar Microtus str. 91001]'
+
+    In this example the two files contain 85 and 10 records respectively.
+
+    BGZF compressed files are supported, and detected automatically. Ordinary
+    GZIP compressed files are not supported.
+
+    See also: Bio.SeqIO.index() and Bio.SeqIO.to_dict()
     """
-    import warnings
-    warnings.warn("The Bio.SeqIO.to_alignment(...) function is deprecated. "
-                  "Please use the Bio.Align.MultipleSeqAlignment(...) object "
-                  "directly instead.", DeprecationWarning)
-    return MultipleSeqAlignment(sequences, alphabet)
+    #Try and give helpful error messages:
+    if not isinstance(index_filename, basestring):
+        raise TypeError("Need a string for the index filename")
+    if isinstance(filenames, basestring):
+        #Make the API a little more friendly, and more similar
+        #to Bio.SeqIO.index(...) for indexing just one file.
+        filenames = [filenames]
+    if filenames is not None and not isinstance(filenames, list):
+        raise TypeError(
+            "Need a list of filenames (as strings), or one filename")
+    if format is not None and not isinstance(format, basestring):
+        raise TypeError("Need a string for the file format (lower case)")
+    if format and format != format.lower():
+        raise ValueError("Format string '%s' should be lower case" % format)
+    if alphabet is not None and not (isinstance(alphabet, Alphabet) or
+                                     isinstance(alphabet, AlphabetEncoder)):
+        raise ValueError("Invalid alphabet, %s" % repr(alphabet))
+
+    #Map the file format to a sequence iterator:
+    from _index import _FormatToRandomAccess  # Lazy import
+    from Bio.File import _SQLiteManySeqFilesDict
+    repr = "SeqIO.index_db(%r, filenames=%r, format=%r, alphabet=%r, key_function=%r)" \
+               % (index_filename, filenames, format, alphabet, key_function)
+
+    def proxy_factory(format, filename=None):
+        """Given a filename returns proxy object, else boolean if format OK."""
+        if filename:
+            return _FormatToRandomAccess[format](filename, format, alphabet)
+        else:
+            return format in _FormatToRandomAccess
+
+    return _SQLiteManySeqFilesDict(index_filename, filenames,
+                                   proxy_factory, format,
+                                   key_function, repr)
+
 
 def convert(in_file, in_format, out_file, out_format, alphabet=None):
     """Convert between two sequence file formats, return number of records.
@@ -818,64 +920,29 @@ def convert(in_file, in_format, out_file, out_format, alphabet=None):
     GTTGCTTCTGGCGTGGGTGGGGGGG
     <BLANKLINE>
     """
-    if isinstance(in_file, basestring):
-        #Hack for SFF, will need to make this more general in future
-        if in_format in _BinaryFormats :
-            in_handle = open(in_file, "rb")
-        else :
-            in_handle = open(in_file, "rU")
-        in_close = True
+    #Hack for SFF, will need to make this more general in future
+    if in_format in _BinaryFormats:
+        in_mode = 'rb'
     else:
-        in_handle = in_file
-        in_close = False
+        in_mode = 'rU'
+
     #Don't open the output file until we've checked the input is OK?
-    if isinstance(out_file, basestring):
-        if out_format in ["sff", "sff_trim"] :
-            out_handle = open(out_file, "wb")
-        else :
-            out_handle = open(out_file, "w")
-        out_close = True
+    if out_format in ["sff", "sff_trim"]:
+        out_mode = 'wb'
     else:
-        out_handle = out_file
-        out_close = False
+        out_mode = 'w'
+
     #This will check the arguments and issue error messages,
     #after we have opened the file which is a shame.
-    from _convert import _handle_convert #Lazy import
-    count = _handle_convert(in_handle, in_format,
-                            out_handle, out_format,
-                            alphabet)
-    #Must now close any handles we opened
-    if in_close:
-        in_handle.close()
-    if out_close:
-        out_handle.close()
+    from _convert import _handle_convert  # Lazy import
+    with as_handle(in_file, in_mode) as in_handle:
+        with as_handle(out_file, out_mode) as out_handle:
+            count = _handle_convert(in_handle, in_format,
+                                    out_handle, out_format,
+                                    alphabet)
     return count
-           
-def _test():
-    """Run the Bio.SeqIO module's doctests.
 
-    This will try and locate the unit tests directory, and run the doctests
-    from there in order that the relative paths used in the examples work.
-    """
-    import doctest
-    import os
-    if os.path.isdir(os.path.join("..", "..", "Tests")):
-        print "Runing doctests..."
-        cur_dir = os.path.abspath(os.curdir)
-        os.chdir(os.path.join("..", "..", "Tests"))
-        doctest.testmod()
-        os.chdir(cur_dir)
-        del cur_dir
-        print "Done"
-    elif os.path.isdir(os.path.join("Tests", "Fasta")):
-        print "Runing doctests..."
-        cur_dir = os.path.abspath(os.curdir)
-        os.chdir(os.path.join("Tests"))
-        doctest.testmod()
-        os.chdir(cur_dir)
-        del cur_dir
-        print "Done"
 
 if __name__ == "__main__":
-    #Run the doctests
-    _test()
+    from Bio._utils import run_doctest
+    run_doctest()
