@@ -1,18 +1,19 @@
 # Copyright 2000-2002 Andrew Dalke.
 # Copyright 2002-2004 Brad Chapman.
-# Copyright 2006-2009 by Peter Cock.
+# Copyright 2006-2010 by Peter Cock.
 # All rights reserved.
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
 """Represent a Sequence Record, a sequence with annotation."""
-__docformat__ = "epytext en" #Simple markup to show doctests nicely
+__docformat__ = "epytext en"  # Simple markup to show doctests nicely
 
 # NEEDS TO BE SYNCH WITH THE REST OF BIOPYTHON AND BIOPERL
 # In particular, the SeqRecord and BioSQL.BioSeq.DBSeqRecord classes
 # need to be in sync (this is the BioSQL "Database SeqRecord", see
 # also BioSQL.BioSeq.DBSeq which is the "Database Seq" class)
 
+
 class _RestrictedDict(dict):
     """Dict which only allows sequences of given length as values (PRIVATE).
 
@@ -23,22 +24,67 @@ class _RestrictedDict(dict):
     expected (the length of the SeqRecord's seq object).  It cannot however
     prevent the entries being edited in situ (for example appending entries
     to a list).
+
+    >>> x = _RestrictedDict(5)
+    >>> x["test"] = "hello"
+    >>> x
+    {'test': 'hello'}
+
+    Adding entries which don't have the expected length are blocked:
+
+    >>> x["test"] = "hello world"
+    Traceback (most recent call last):
+    ...
+    TypeError: We only allow python sequences (lists, tuples or strings) of length 5.
+
+    The expected length is stored as a private attribute,
+
+    >>> x._length
+    5
+
+    In order that the SeqRecord (and other objects using this class) can be
+    pickled, for example for use in the multiprocessing library, we need to
+    be able to pickle the restricted dictionary objects.
+
+    Using the default protocol, which is 0 on Python 2.x,
+
+    >>> import pickle
+    >>> y = pickle.loads(pickle.dumps(x))
+    >>> y
+    {'test': 'hello'}
+    >>> y._length
+    5
+
+    Using the highest protocol, which is 2 on Python 2.x,
+
+    >>> import pickle
+    >>> z = pickle.loads(pickle.dumps(x, pickle.HIGHEST_PROTOCOL))
+    >>> z
+    {'test': 'hello'}
+    >>> z._length
+    5
     """
+
     def __init__(self, length):
         """Create an EMPTY restricted dictionary."""
         dict.__init__(self)
         self._length = int(length)
+
     def __setitem__(self, key, value):
-        if not hasattr(value,"__len__") or not hasattr(value,"__getitem__") \
-        or len(value) != self._length:
+        #The check hasattr(self, "_length") is to cope with pickle protocol 2
+        #I couldn't seem to avoid this with __getstate__ and __setstate__
+        if not hasattr(value, "__len__") or not hasattr(value, "__getitem__") \
+        or (hasattr(self, "_length") and len(value) != self._length):
             raise TypeError("We only allow python sequences (lists, tuples or "
                             "strings) of length %i." % self._length)
         dict.__setitem__(self, key, value)
+
     def update(self, new_dict):
         #Force this to go via our strict __setitem__ method
         for (key, value) in new_dict.iteritems():
             self[key] = value
 
+
 class SeqRecord(object):
     """A SeqRecord object holds a sequence and information about it.
 
@@ -51,7 +97,7 @@ class SeqRecord(object):
      - description - Additional text (string)
      - dbxrefs     - List of database cross references (list of strings)
      - features    - Any (sub)features defined (list of SeqFeature objects)
-     - annotations - Further information about the whole sequence (dictionary)
+     - annotations - Further information about the whole sequence (dictionary).
                      Most entries are strings, or lists of strings.
      - letter_annotations - Per letter/symbol annotation (restricted
                      dictionary). This holds Python sequences (lists, strings
@@ -97,7 +143,7 @@ class SeqRecord(object):
     MKQHKAMIVAIVICITAVVAALVTRKDLCEVHIRTGQTEVAVF
     >>> print record.seq
     MKQHKAMIVALIVICITAVVAALVTRKDLCEVHIRTGQTEVAVF
-    
+
     """
     def __init__(self, seq, id = "<unknown id>", name = "<unknown name>",
                  description = "<unknown description>", dbxrefs = None,
@@ -129,7 +175,7 @@ class SeqRecord(object):
         then using the UnknownSeq object from Bio.Seq is appropriate.
 
         You can create a 'blank' SeqRecord object, and then populate the
-        attributes later.  
+        attributes later.
         """
         if id is not None and not isinstance(id, basestring):
             #Lots of existing code uses id=None... this may be a bad idea.
@@ -149,7 +195,7 @@ class SeqRecord(object):
         elif not isinstance(dbxrefs, list):
             raise TypeError("dbxrefs argument should be a list (of strings)")
         self.dbxrefs = dbxrefs
-        
+
         # annotations about the whole sequence
         if annotations is None:
             annotations = {}
@@ -173,7 +219,7 @@ class SeqRecord(object):
             #turn this into a _RestrictedDict and thus ensure all the values
             #in the dict are the right length
             self.letter_annotations = letter_annotations
-        
+
         # annotations about parts of the sequence
         if features is None:
             features = []
@@ -193,8 +239,8 @@ class SeqRecord(object):
             #e.g. seq is None
             self._per_letter_annotations = _RestrictedDict(length=0)
         self._per_letter_annotations.update(value)
-    letter_annotations = property( \
-        fget=lambda self : self._per_letter_annotations,
+    letter_annotations = property(
+        fget=lambda self: self._per_letter_annotations,
         fset=_set_per_letter_annotations,
         doc="""Dictionary of per-letter-annotation for the sequence.
 
@@ -203,9 +249,7 @@ class SeqRecord(object):
         variant FASTQ file as a SeqRecord:
 
         >>> from Bio import SeqIO
-        >>> handle = open("Quality/solexa_faked.fastq", "rU")
-        >>> record = SeqIO.read(handle, "fastq-solexa")
-        >>> handle.close()
+        >>> record = SeqIO.read("Quality/solexa_faked.fastq", "fastq-solexa")
         >>> print record.id, record.seq
         slxa_0001_1_0001_01 ACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTNNNNNN
         >>> print record.letter_annotations.keys()
@@ -257,7 +301,7 @@ class SeqRecord(object):
             #e.g. seq is None
             self._per_letter_annotations = _RestrictedDict(length=0)
 
-    seq = property(fget=lambda self : self._seq,
+    seq = property(fget=lambda self: self._seq,
                    fset=_set_seq,
                    doc="The sequence itself, as a Seq or MutableSeq object.")
 
@@ -385,31 +429,17 @@ class SeqRecord(object):
             #TODO - The desription may no longer apply.
             #It would be safer to change it to something
             #generic like "edited" or the default value.
-            
+
             #Don't copy the annotation dict and dbxefs list,
             #they may not apply to a subsequence.
             #answer.annotations = dict(self.annotations.iteritems())
             #answer.dbxrefs = self.dbxrefs[:]
             #TODO - Review this in light of adding SeqRecord objects?
-            
+
             #TODO - Cope with strides by generating ambiguous locations?
-            if index.step is None or index.step == 1:
+            start, stop, step = index.indices(parent_length)
+            if step == 1:
                 #Select relevant features, add them with shifted locations
-                if index.start is None:
-                    start = 0
-                else:
-                    start = index.start
-                if index.stop is None:
-                    stop = -1
-                else:
-                    stop = index.stop
-                if (start < 0 or stop < 0) and parent_length == 0:
-                    raise ValueError, \
-                          "Cannot support negative indices without the sequence length"
-                if start < 0:
-                    start = parent_length + start
-                if stop < 0:
-                    stop  = parent_length + stop + 1
                 #assert str(self.seq)[index] == str(self.seq)[start:stop]
                 for f in self.features:
                     if f.ref or f.ref_db:
@@ -417,7 +447,7 @@ class SeqRecord(object):
                         import warnings
                         warnings.warn("When slicing SeqRecord objects, any "
                               "SeqFeature referencing other sequences (e.g. "
-                              "from segmented GenBank records) is ignored.")
+                              "from segmented GenBank records) are ignored.")
                         continue
                     if start <= f.location.nofuzzy_start \
                     and f.location.nofuzzy_end <= stop:
@@ -429,7 +459,7 @@ class SeqRecord(object):
                 answer._per_letter_annotations[key] = value[index]
 
             return answer
-        raise ValueError, "Invalid index"
+        raise ValueError("Invalid index")
 
     def __iter__(self):
         """Iterate over the letters in the sequence.
@@ -437,10 +467,10 @@ class SeqRecord(object):
         For example, using Bio.SeqIO to read in a protein FASTA file:
 
         >>> from Bio import SeqIO
-        >>> record = SeqIO.read(open("Fasta/loveliesbleeding.pro"),"fasta")
+        >>> record = SeqIO.read("Fasta/loveliesbleeding.pro", "fasta")
         >>> for amino in record:
         ...     print amino
-        ...     if amino == "L" : break
+        ...     if amino == "L": break
         X
         A
         G
@@ -452,22 +482,21 @@ class SeqRecord(object):
 
         >>> for amino in record.seq:
         ...     print amino
-        ...     if amino == "L" : break
+        ...     if amino == "L": break
         X
         A
         G
         L
         >>> print record.seq[3]
         L
-        
+
         Note that this does not facilitate iteration together with any
         per-letter-annotation.  However, you can achieve that using the
         python zip function on the record (or its sequence) and the relevant
         per-letter-annotation:
-        
+
         >>> from Bio import SeqIO
-        >>> rec = SeqIO.read(open("Quality/solexa_faked.fastq", "rU"),
-        ...                  "fastq-solexa")
+        >>> rec = SeqIO.read("Quality/solexa_faked.fastq", "fastq-solexa")
         >>> print rec.id, rec.seq
         slxa_0001_1_0001_01 ACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTNNNNNN
         >>> print rec.letter_annotations.keys()
@@ -492,7 +521,7 @@ class SeqRecord(object):
         e.g.
 
         >>> from Bio import SeqIO
-        >>> record = SeqIO.read(open("Fasta/sweetpea.nu"), "fasta")
+        >>> record = SeqIO.read("Fasta/sweetpea.nu", "fasta")
         >>> "GAATTC" in record
         False
         >>> "AAA" in record
@@ -515,10 +544,9 @@ class SeqRecord(object):
         True
 
         See also the Seq object's __contains__ method.
-        """        
+        """
         return char in self.seq
 
-
     def __str__(self):
         """A human readable summary of the record and its annotation (string).
 
@@ -552,16 +580,20 @@ class SeqRecord(object):
         Note that long sequences are shown truncated.
         """
         lines = []
-        if self.id : lines.append("ID: %s" % self.id)
-        if self.name : lines.append("Name: %s" % self.name)
-        if self.description : lines.append("Description: %s" % self.description)
-        if self.dbxrefs : lines.append("Database cross-references: " \
-                                       + ", ".join(self.dbxrefs))
+        if self.id:
+            lines.append("ID: %s" % self.id)
+        if self.name:
+            lines.append("Name: %s" % self.name)
+        if self.description:
+            lines.append("Description: %s" % self.description)
+        if self.dbxrefs:
+            lines.append("Database cross-references: "
+                         + ", ".join(self.dbxrefs))
         lines.append("Number of features: %i" % len(self.features))
         for a in self.annotations:
             lines.append("/%s=%s" % (a, str(self.annotations[a])))
         if self.letter_annotations:
-            lines.append("Per letter annotation for: " \
+            lines.append("Per letter annotation for: "
                          + ", ".join(self.letter_annotations.keys()))
         #Don't want to include the entire sequence,
         #and showing the alphabet is useful:
@@ -627,7 +659,7 @@ class SeqRecord(object):
         in this example a blank line is shown.  If you look at the string
         representation you can see there is a trailing new line (shown as
         slash n) which is important when writing to a file or if
-        concatenating mutliple sequence strings together.
+        concatenating multiple sequence strings together.
 
         Note that this method will NOT work on every possible file format
         supported by Bio.SeqIO (e.g. some are for multiple sequences only).
@@ -646,7 +678,7 @@ class SeqRecord(object):
         """
         if not format_spec:
             #Follow python convention and default to using __str__
-            return str(self)    
+            return str(self)
         from Bio import SeqIO
         if format_spec in SeqIO._BinaryFormats:
             #Return bytes on Python 3
@@ -670,7 +702,7 @@ class SeqRecord(object):
         For example, using Bio.SeqIO to read in a FASTA nucleotide file:
 
         >>> from Bio import SeqIO
-        >>> record = SeqIO.read(open("Fasta/sweetpea.nu"),"fasta")
+        >>> record = SeqIO.read("Fasta/sweetpea.nu", "fasta")
         >>> len(record)
         309
         >>> len(record.seq)
@@ -703,9 +735,7 @@ class SeqRecord(object):
         annotation will be lost:
 
         >>> from Bio import SeqIO
-        >>> handle = open("Quality/solexa_faked.fastq", "rU")
-        >>> record = SeqIO.read(handle, "fastq-solexa")
-        >>> handle.close()
+        >>> record = SeqIO.read("Quality/solexa_faked.fastq", "fastq-solexa")
         >>> print record.id, record.seq
         slxa_0001_1_0001_01 ACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTNNNNNN
         >>> print record.letter_annotations.keys()
@@ -716,7 +746,7 @@ class SeqRecord(object):
         slxa_0001_1_0001_01 ACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTNNNNNNACT
         >>> print new.letter_annotations.keys()
         []
-        
+
         The new record will attempt to combine the annotation, but for any
         ambiguities (e.g. different names) it defaults to omitting that
         annotation.
@@ -745,7 +775,7 @@ class SeqRecord(object):
 
         When we add the left and right SeqRecord objects, their annotation
         is all consistent, so it is all conserved in the new SeqRecord:
-        
+
         >>> new.id == left.id == right.id == plasmid.id
         True
         >>> new.name == left.name == right.name == plasmid.name
@@ -786,7 +816,7 @@ class SeqRecord(object):
         del l
         for ref in other.dbxrefs:
             if ref not in answer.dbxrefs:
-                answer.append(ref)
+                answer.dbxrefs.append(ref)
         #Take common id/name/description/annotation
         if self.id == other.id:
             answer.id = self.id
@@ -794,15 +824,15 @@ class SeqRecord(object):
             answer.name = self.name
         if self.description == other.description:
             answer.description = self.description
-        for k,v in self.annotations.iteritems():
+        for k, v in self.annotations.iteritems():
             if k in other.annotations and other.annotations[k] == v:
                 answer.annotations[k] = v
         #Can append matching per-letter-annotation
-        for k,v in self.letter_annotations.iteritems():
+        for k, v in self.letter_annotations.iteritems():
             if k in other.letter_annotations:
                 answer.letter_annotations[k] = v + other.letter_annotations[k]
         return answer
-        
+
     def __radd__(self, other):
         """Add another sequence or string to this sequence (from the left).
 
@@ -811,9 +841,7 @@ class SeqRecord(object):
         See the __add__ method for more details, but for example:
 
         >>> from Bio import SeqIO
-        >>> handle = open("Quality/solexa_faked.fastq", "rU")
-        >>> record = SeqIO.read(handle, "fastq-solexa")
-        >>> handle.close()
+        >>> record = SeqIO.read("Quality/solexa_faked.fastq", "fastq-solexa")
         >>> print record.id, record.seq
         slxa_0001_1_0001_01 ACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTNNNNNN
         >>> print record.letter_annotations.keys()
@@ -857,7 +885,7 @@ class SeqRecord(object):
         <BLANKLINE>
 
         Naturally, there is a matching lower method:
-        
+
         >>> print record.lower().format("fastq")
         @Test Made up for this example
         acgtacgt
@@ -913,30 +941,180 @@ class SeqRecord(object):
                          annotations = self.annotations.copy(),
                          letter_annotations=self.letter_annotations.copy())
 
-def _test():
-    """Run the Bio.SeqRecord module's doctests (PRIVATE).
+    def reverse_complement(self, id=False, name=False, description=False,
+                           features=True, annotations=False,
+                           letter_annotations=True, dbxrefs=False):
+        """Returns new SeqRecord with reverse complement sequence.
+
+        You can specify the returned record's id, name and description as
+        strings, or True to keep that of the parent, or False for a default.
+
+        You can specify the returned record's features with a list of
+        SeqFeature objects, or True to keep that of the parent, or False to
+        omit them. The default is to keep the original features (with the
+        strand and locations adjusted).
+
+        You can also specify both the returned record's annotations and
+        letter_annotations as dictionaries, True to keep that of the parent,
+        or False to omit them. The default is to keep the original
+        annotations (with the letter annotations reversed).
+
+        To show what happens to the pre-letter annotations, consider an
+        example Solexa variant FASTQ file with a single entry, which we'll
+        read in as a SeqRecord:
+
+        >>> from Bio import SeqIO
+        >>> record = SeqIO.read("Quality/solexa_faked.fastq", "fastq-solexa")
+        >>> print record.id, record.seq
+        slxa_0001_1_0001_01 ACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTNNNNNN
+        >>> print record.letter_annotations.keys()
+        ['solexa_quality']
+        >>> print record.letter_annotations["solexa_quality"]
+        [40, 39, 38, 37, 36, 35, 34, 33, 32, 31, 30, 29, 28, 27, 26, 25, 24, 23, 22, 21, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0, -1, -2, -3, -4, -5]
+
+        Now take the reverse complement,
+
+        >>> rc_record = record.reverse_complement(id=record.id+"_rc")
+        >>> print rc_record.id, rc_record.seq
+        slxa_0001_1_0001_01_rc NNNNNNACGTACGTACGTACGTACGTACGTACGTACGTACGTACGT
+
+        Notice that the per-letter-annotations have also been reversed,
+        although this may not be appropriate for all cases.
+
+        >>> print rc_record.letter_annotations["solexa_quality"]
+        [-5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40]
+
+        Now for the features, we need a different example. Parsing a GenBank
+        file is probably the easiest way to get an nice example with features
+        in it...
+
+        >>> from Bio import SeqIO
+        >>> handle = open("GenBank/pBAD30.gb")
+        >>> plasmid = SeqIO.read(handle, "gb")
+        >>> handle.close()
+        >>> print plasmid.id, len(plasmid)
+        pBAD30 4923
+        >>> plasmid.seq
+        Seq('GCTAGCGGAGTGTATACTGGCTTACTATGTTGGCACTGATGAGGGTGTCAGTGA...ATG', IUPACAmbiguousDNA())
+        >>> len(plasmid.features)
+        13
+
+        Now, let's take the reverse complement of this whole plasmid:
+
+        >>> rc_plasmid = plasmid.reverse_complement(id=plasmid.id+"_rc")
+        >>> print rc_plasmid.id, len(rc_plasmid)
+        pBAD30_rc 4923
+        >>> rc_plasmid.seq
+        Seq('CATGGGCAAATATTATACGCAAGGCGACAAGGTGCTGATGCCGCTGGCGATTCA...AGC', IUPACAmbiguousDNA())
+        >>> len(rc_plasmid.features)
+        13
+
+        Let's compare the first CDS feature - it has gone from being the
+        second feature (index 1) to the second last feature (index -2), its
+        strand has changed, and the location switched round.
+
+        >>> print plasmid.features[1]
+        type: CDS
+        location: [1081:1960](-)
+        qualifiers: 
+            Key: label, Value: ['araC']
+            Key: note, Value: ['araC regulator of the arabinose BAD promoter']
+            Key: vntifkey, Value: ['4']
+        <BLANKLINE>
+        >>> print rc_plasmid.features[-2]
+        type: CDS
+        location: [2963:3842](+)
+        qualifiers: 
+            Key: label, Value: ['araC']
+            Key: note, Value: ['araC regulator of the arabinose BAD promoter']
+            Key: vntifkey, Value: ['4']
+        <BLANKLINE>
+
+        You can check this new location, based on the length of the plasmid:
+
+        >>> len(plasmid) - 1081
+        3842
+        >>> len(plasmid) - 1960
+        2963
+
+        Note that if the SeqFeature annotation includes any strand specific
+        information (e.g. base changes for a SNP), this information is not
+        ammended, and would need correction after the reverse complement.
+
+        Note trying to reverse complement a protein SeqRecord raises an
+        exception:
+
+        >>> from Bio.SeqRecord import SeqRecord
+        >>> from Bio.Seq import Seq
+        >>> from Bio.Alphabet import IUPAC
+        >>> protein_rec = SeqRecord(Seq("MAIVMGR", IUPAC.protein), id="Test")
+        >>> protein_rec.reverse_complement()
+        Traceback (most recent call last):
+           ...
+        ValueError: Proteins do not have complements!
+
+        Also note you can reverse complement a SeqRecord using a MutableSeq:
+
+        >>> from Bio.SeqRecord import SeqRecord
+        >>> from Bio.Seq import MutableSeq
+        >>> from Bio.Alphabet import generic_dna
+        >>> rec = SeqRecord(MutableSeq("ACGT", generic_dna), id="Test")
+        >>> rec.seq[0] = "T"
+        >>> print rec.id, rec.seq
+        Test TCGT
+        >>> rc = rec.reverse_complement(id=True)
+        >>> print rc.id, rc.seq
+        Test ACGA
+        """
+        from Bio.Seq import MutableSeq  # Lazy to avoid circular imports
+        if isinstance(self.seq, MutableSeq):
+            #Currently the MutableSeq reverse complement is in situ
+            answer = SeqRecord(self.seq.toseq().reverse_complement())
+        else:
+            answer = SeqRecord(self.seq.reverse_complement())
+        if isinstance(id, basestring):
+            answer.id = id
+        elif id:
+            answer.id = self.id
+        if isinstance(name, basestring):
+            answer.name = name
+        elif name:
+            answer.name = self.name
+        if isinstance(description, basestring):
+            answer.description = description
+        elif description:
+            answer.description = self.description
+        if isinstance(dbxrefs, list):
+            answer.dbxrefs = dbxrefs
+        elif dbxrefs:
+            #Copy the old dbxrefs
+            answer.dbxrefs = self.dbxrefs[:]
+        if isinstance(features, list):
+            answer.features = features
+        elif features:
+            #Copy the old features, adjusting location and string
+            l = len(answer)
+            answer.features = [f._flip(l) for f in self.features]
+            #The old list should have been sorted by start location,
+            #reversing it will leave it sorted by what is now the end position,
+            #so we need to resort in case of overlapping features.
+            #NOTE - In the common case of gene before CDS (and similar) with
+            #the exact same locations, this will still maintain gene before CDS
+            answer.features.sort(key=lambda x: x.location.start.position)
+        if isinstance(annotations, dict):
+            answer.annotations = annotations
+        elif annotations:
+            #Copy the old annotations,
+            answer.annotations = self.annotations.copy()
+        if isinstance(letter_annotations, dict):
+            answer.letter_annotations = letter_annotations
+        elif letter_annotations:
+            #Copy the old per letter annotations, reversing them
+            for key, value in self.letter_annotations.iteritems():
+                answer._per_letter_annotations[key] = value[::-1]
+        return answer
 
-    This will try and locate the unit tests directory, and run the doctests
-    from there in order that the relative paths used in the examples work.
-    """
-    import doctest
-    import os
-    if os.path.isdir(os.path.join("..","Tests")):
-        print "Runing doctests..."
-        cur_dir = os.path.abspath(os.curdir)
-        os.chdir(os.path.join("..","Tests"))
-        doctest.testmod()
-        os.chdir(cur_dir)
-        del cur_dir
-        print "Done"
-    elif os.path.isdir(os.path.join("Tests")) :
-        print "Runing doctests..."
-        cur_dir = os.path.abspath(os.curdir)
-        os.chdir(os.path.join("Tests"))
-        doctest.testmod()
-        os.chdir(cur_dir)
-        del cur_dir
-        print "Done"
 
 if __name__ == "__main__":
-    _test()
+    from Bio._utils import run_doctest
+    run_doctest()
