@@ -1,6 +1,7 @@
 # Copyright 2002 by Andrew Dalke.  All rights reserved.
 # Revisions 2007-2009 copyright by Peter Cock.  All rights reserved.
 # Revisions 2009 copyright by Cymon J. Cox.  All rights reserved.
+# Revisions 2013 copyright by Tiago Antao.  All rights reserved.
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
@@ -12,18 +13,23 @@
 This provides interfaces for loading biological objects from a relational
 database, and is compatible with the BioSQL standards.
 """
+import os
+
+from Bio import BiopythonDeprecationWarning
+
 import BioSeq
 import Loader
 import DBUtils
 
-_POSTGRES_RULES_PRESENT = False # Hack for BioSQL Bug 2839
+_POSTGRES_RULES_PRESENT = False  # Hack for BioSQL Bug 2839
 
-def open_database(driver = "MySQLdb", **kwargs):
+
+def open_database(driver="MySQLdb", **kwargs):
     """Main interface for loading a existing BioSQL-style database.
 
     This function is the easiest way to retrieve a connection to a
     database, doing something like:
-        
+
         >>> from BioSeq import BioSeqDatabase
         >>> server = BioSeqDatabase.open_database(user="root", db="minidb")
 
@@ -40,12 +46,23 @@ def open_database(driver = "MySQLdb", **kwargs):
         raise ValueError("Using BioSQL with psycopg (version one) is no "
                          "longer supported. Use psycopg2 instead.")
 
-    module = __import__(driver)
+    if os.name == "java":
+        from com.ziclix.python.sql import zxJDBC
+        module = zxJDBC
+        if driver in ["MySQLdb"]:
+            jdbc_driver = "com.mysql.jdbc.Driver"
+            url_pref = "jdbc:mysql://" + kwargs["host"] + "/"
+        elif driver in ["psycopg2"]:
+            jdbc_driver = "org.postgresql.Driver"
+            url_pref = "jdbc:postgresql://" + kwargs["host"] + "/"
+
+    else:
+        module = __import__(driver)
     connect = getattr(module, "connect")
 
     # Different drivers use different keywords...
     kw = kwargs.copy()
-    if driver == "MySQLdb":
+    if driver == "MySQLdb" and os.name != "java":
         if "database" in kw:
             kw["db"] = kw["database"]
             del kw["database"]
@@ -63,7 +80,15 @@ def open_database(driver = "MySQLdb", **kwargs):
     if driver in ["psycopg2", "pgdb"] and not kw.get("database"):
         kw["database"] = "template1"
     # SQLite connect takes the database name as input
-    if driver in ["sqlite3"]:
+    if os.name == "java":
+        if driver in ["MySQLdb"]:
+            conn = connect(url_pref + kw.get("database", "mysql"),
+                           kw["user"], kw["password"], jdbc_driver)
+        elif driver in ["psycopg2"]:
+            conn = connect(url_pref + kw.get("database", "postgresql") +
+                           "?stringtype=unspecified",
+                           kw["user"], kw["password"], jdbc_driver)
+    elif driver in ["sqlite3"]:
         conn = connect(kw["database"])
     else:
         try:
@@ -80,7 +105,10 @@ def open_database(driver = "MySQLdb", **kwargs):
             dsn = ' '.join(['='.join(i) for i in kw.items()])
             conn = connect(dsn)
 
-    server = DBServer(conn, module)
+    if os.name == "java":
+        server = DBServer(conn, module, driver)
+    else:
+        server = DBServer(conn, module)
 
     # TODO - Remove the following once BioSQL Bug 2839 is fixed.
     # Test for RULES in PostgreSQL schema, see also Bug 2833.
@@ -90,6 +118,7 @@ def open_database(driver = "MySQLdb", **kwargs):
               "rulename='rule_bioentry_i2';"
         if server.adaptor.execute_and_fetchall(sql):
             import warnings
+            from Bio import BiopythonWarning
             warnings.warn("Your BioSQL PostgreSQL schema includes some "
                           "rules currently required for bioperl-db but "
                           "which may cause problems loading data using "
@@ -97,15 +126,16 @@ def open_database(driver = "MySQLdb", **kwargs):
                           "use BioPerl, please remove these rules. "
                           "Biopython should cope with the rules present, "
                           "but with a performance penalty when loading "
-                          "new records.")
+                          "new records.", BiopythonWarning)
             global _POSTGRES_RULES_PRESENT
             _POSTGRES_RULES_PRESENT = True
 
     return server
 
+
 class DBServer:
     """Represents a BioSQL database continaing namespaces (sub-databases).
-    
+
     This acts like a Python dictionary, giving access to each namespace
     (defined by a row in the biodatabase table) as a BioSeqDatabase object.
     """
@@ -115,7 +145,7 @@ class DBServer:
             module_name = module.__name__
         self.adaptor = Adaptor(conn, DBUtils.get_dbutils(module_name))
         self.module_name = module_name
-        
+
     def __repr__(self):
         return self.__class__.__name__ + "(%r)" % self.adaptor.conn
 
@@ -131,14 +161,14 @@ class DBServer:
         """Check if a namespace (sub-database) in this database."""
         sql = "SELECT COUNT(name) FROM biodatabase WHERE name=%s;"
         return bool(self.adaptor.execute_and_fetch_col0(sql, (value,))[0])
-    
+
     def __iter__(self):
         """Iterate over namespaces (sub-databases) in the database."""
         #TODO - Iterate over the cursor, much more efficient
-        return iter(self.adaptor.list_biodatabase_names())        
+        return iter(self.adaptor.list_biodatabase_names())
 
     if hasattr(dict, "iteritems"):
-        #Python 2, use iteritems etc    
+        #Python 2, use iteritems etc
         def keys(self):
             """List of namespaces (sub-databases) in the database."""
             return self.adaptor.list_biodatabase_names()
@@ -146,20 +176,20 @@ class DBServer:
         def values(self):
             """List of BioSeqDatabase objects in the database."""
             return [self[key] for key in self.keys()]
-    
+
         def items(self):
             """List of (namespace, BioSeqDatabase) for entries in the database."""
             return [(key, self[key]) for key in self.keys()]
-        
+
         def iterkeys(self):
             """Iterate over namespaces (sub-databases) in the database."""
             return iter(self)
-    
+
         def itervalues(self):
             """Iterate over BioSeqDatabase objects in the database."""
             for key in self:
                 yield self[key]
-            
+
         def iteritems(self):
             """Iterate over (namespace, BioSeqDatabase) in the database."""
             for key in self:
@@ -169,12 +199,12 @@ class DBServer:
         def keys(self):
             """Iterate over namespaces (sub-databases) in the database."""
             return iter(self)
-            
+
         def values(self):
             """Iterate over BioSeqDatabase objects in the database."""
             for key in self:
                 yield self[key]
-    
+
         def items(self):
             """Iterate over (namespace, BioSeqDatabase) in the database."""
             for key in self:
@@ -188,15 +218,19 @@ class DBServer:
 
     def remove_database(self, db_name):
         """Remove a namespace and all its entries (OBSOLETE).
-        
+
         Try to remove all references to items in a database.
-        
+
         server.remove_database(name)
-        
+
         In keeping with the dictionary interface, you can now do this:
-        
+
         del server[name]
         """
+        import warnings
+        warnings.warn("This method is deprecated.  In keeping with the "
+                      "dictionary interface, you can now use 'del "
+                      "server[name]' instead", BiopythonDeprecationWarning)
         db_id = self.adaptor.fetch_dbid_by_dbname(db_name)
         remover = Loader.DatabaseRemover(self.adaptor, db_id)
         remover.remove()
@@ -206,8 +240,8 @@ class DBServer:
         """
         # make the database
         sql = r"INSERT INTO biodatabase (name, authority, description)" \
-              r" VALUES (%s, %s, %s)" 
-        self.adaptor.execute(sql, (db_name,authority, description))
+              r" VALUES (%s, %s, %s)"
+        self.adaptor.execute(sql, (db_name, authority, description))
         return BioSeqDatabase(self.adaptor, db_name)
 
     def load_database_sql(self, sql_file):
@@ -225,14 +259,14 @@ class DBServer:
         sql_handle = open(sql_file, "rU")
         sql = r""
         for line in sql_handle:
-            if line.find("--") == 0: # don't include comment lines
+            if line.startswith("--"):  # don't include comment lines
                 pass
-            elif line.find("#") == 0: # ditto for MySQL comments
+            elif line.startswith("#"):  # ditto for MySQL comments
                 pass
-            elif line.strip(): # only include non-blank lines
+            elif line.strip():  # only include non-blank lines
                 sql += line.strip()
                 sql += ' '
-        
+
         # two ways to load the SQL
         # 1. PostgreSQL can load it all at once and actually needs to
         # due to FUNCTION defines at the end of the SQL which mess up
@@ -242,8 +276,8 @@ class DBServer:
         # 2. MySQL needs the database loading split up into single lines of
         # SQL executed one at a time
         elif self.module_name in ["MySQLdb", "sqlite3"]:
-            sql_parts = sql.split(";") # one line per sql command
-            for sql_line in sql_parts[:-1]: # don't use the last item, it's blank
+            sql_parts = sql.split(";")  # one line per sql command
+            for sql_line in sql_parts[:-1]:  # don't use the last item, it's blank
                 self.adaptor.cursor.execute(sql_line)
         else:
             raise ValueError("Module %s not supported by the loader." %
@@ -261,6 +295,7 @@ class DBServer:
         """Close the connection. No further activity possible."""
         return self.adaptor.close()
 
+
 class Adaptor:
     def __init__(self, conn, dbutils):
         self.conn = conn
@@ -385,7 +420,7 @@ class Adaptor:
 
     def list_any_ids(self, sql, args):
         """Return ids given a SQL statement to select for them.
-        
+
         This assumes that the given SQL does a SELECT statement that
         returns a list of items. This parses them out of the 2D list
         they come as and just returns them in a list.
@@ -401,6 +436,8 @@ class Adaptor:
     def execute(self, sql, args=None):
         """Just execute an sql command.
         """
+        if os.name == "java":
+            sql = sql.replace("%s", "?")
         self.dbutils.execute(self.cursor, sql, args)
 
     def get_subseq_as_string(self, seqid, start, end):
@@ -411,13 +448,13 @@ class Adaptor:
         #    """select SUBSTRING(seq FROM %s FOR %s)
         #             from biosequence where bioentry_id = %s""",
         #    (start+1, length, seqid))[0]
-        # 
+        #
         # Convert to a string on returning for databases that give back
         # unicode. Shouldn't need unicode for sequences so this seems safe.
         return str(self.execute_one(
             """select SUBSTR(seq, %s, %s)
                      from biosequence where bioentry_id = %s""",
-            (start+1, length, seqid))[0])
+            (start + 1, length, seqid))[0])
 
     def execute_and_fetch_col0(self, sql, args=None):
         self.execute(sql, args or ())
@@ -437,9 +474,10 @@ _allowed_lookups = {
     'version':    "fetch_seqid_by_version",
     }
 
+
 class BioSeqDatabase:
     """Represents a namespace (sub-database) within the BioSQL database.
-    
+
     i.e. One row in the biodatabase table, and all all rows in the bioentry
     table associated with it.
     """
@@ -450,12 +488,12 @@ class BioSeqDatabase:
 
     def __repr__(self):
         return "BioSeqDatabase(%r, %r)" % (self.adaptor, self.name)
-        
+
     def get_Seq_by_id(self, name):
         """Gets a DBSeqRecord object by its name
 
         Example: seq_rec = db.get_Seq_by_id('ROA1_HUMAN')
-        
+
         The name of this method is misleading since it returns a DBSeqRecord
         rather than a DBSeq ojbect, and presumably was to mirror BioPerl.
         """
@@ -502,13 +540,13 @@ class BioSeqDatabase:
         These maybe ids (display style) or accession numbers or
         something else completely different - they *are not*
         meaningful outside of this database implementation.
-        
+
         Please use .keys() instead of .get_all_primary_ids()
         """
         import warnings
         warnings.warn("Use bio_seq_database.keys() instead of "
                       "bio_seq_database.get_all_primary_ids()",
-                      PendingDeprecationWarning)
+                      BiopythonDeprecationWarning)
         return self.keys()
 
     def __getitem__(self, key):
@@ -521,13 +559,13 @@ class BioSeqDatabase:
         #Assuming this will automatically cascade to the other tables...
         sql = "DELETE FROM bioentry " + \
               "WHERE biodatabase_id=%s AND bioentry_id=%s;"
-        self.adaptor.execute(sql, (self.dbid,key))
+        self.adaptor.execute(sql, (self.dbid, key))
 
     def __len__(self):
         """Number of records in this namespace (sub database)."""
         sql = "SELECT COUNT(bioentry_id) FROM bioentry " + \
               "WHERE biodatabase_id=%s;"
-        return int(self.adaptor.execute_and_fetch_col0(sql, (self.dbid,))[0])
+        return int(self.adaptor.execute_and_fetch_col0(sql, (self.dbid, ))[0])
 
     def __contains__(self, value):
         """Check if a primary (internal) id is this namespace (sub database)."""
@@ -541,14 +579,14 @@ class BioSeqDatabase:
             return False
         return bool(self.adaptor.execute_and_fetch_col0(sql,
                                                   (self.dbid, bioentry_id))[0])
-    
+
     def __iter__(self):
         """Iterate over ids (which may not be meaningful outside this database)."""
         #TODO - Iterate over the cursor, much more efficient
-        return iter(self.adaptor.list_bioentry_ids(self.dbid))        
+        return iter(self.adaptor.list_bioentry_ids(self.dbid))
 
     if hasattr(dict, "iteritems"):
-        #Python 2, use iteritems etc    
+        #Python 2, use iteritems etc
         def keys(self):
             """List of ids which may not be meaningful outside this database."""
             return self.adaptor.list_bioentry_ids(self.dbid)
@@ -556,20 +594,20 @@ class BioSeqDatabase:
         def values(self):
             """List of DBSeqRecord objects in the namespace (sub database)."""
             return [self[key] for key in self.keys()]
-    
+
         def items(self):
             """List of (id, DBSeqRecord) for the namespace (sub database)."""
             return [(key, self[key]) for key in self.keys()]
-        
+
         def iterkeys(self):
             """Iterate over ids (which may not be meaningful outside this database)."""
             return iter(self)
-    
+
         def itervalues(self):
             """Iterate over DBSeqRecord objects in the namespace (sub database)."""
             for key in self:
                 yield self[key]
-            
+
         def iteritems(self):
             """Iterate over (id, DBSeqRecord) for the namespace (sub database)."""
             for key in self:
@@ -579,12 +617,12 @@ class BioSeqDatabase:
         def keys(self):
             """Iterate over ids (which may not be meaningful outside this database)."""
             return iter(self)
-            
+
         def values(self):
             """Iterate over DBSeqRecord objects in the namespace (sub database)."""
             for key in self:
                 yield self[key]
-    
+
         def items(self):
             """Iterate over (id, DBSeqRecord) for the namespace (sub database)."""
             for key in self:
@@ -595,25 +633,25 @@ class BioSeqDatabase:
             raise TypeError("single key/value parameter expected")
         k, v = kwargs.items()[0]
         if k not in _allowed_lookups:
-            raise TypeError("lookup() expects one of %s, not %r" % \
+            raise TypeError("lookup() expects one of %s, not %r" %
                             (repr(_allowed_lookups.keys())[1:-1], repr(k)))
         lookup_name = _allowed_lookups[k]
         lookup_func = getattr(self.adaptor, lookup_name)
         seqid = lookup_func(self.dbid, v)
         return BioSeq.DBSeqRecord(self.adaptor, seqid)
-        
+
     def get_Seq_by_primary_id(self, seqid):
         """Get a DBSeqRecord by the primary (internal) id (OBSOLETE).
-        
+
         Rather than db.get_Seq_by_primary_id(my_id) use db[my_id]
-        
+
         The name of this method is misleading since it returns a DBSeqRecord
         rather than a DBSeq ojbect, and presumably was to mirror BioPerl.
         """
         import warnings
         warnings.warn("Use bio_seq_database[my_id] instead of "
                       "bio_seq_database.get_Seq_by_primary_id(my_id)",
-                      PendingDeprecationWarning)
+                      BiopythonDeprecationWarning)
         return self[seqid]
 
     def load(self, record_iterator, fetch_NCBI_taxonomy=False):
@@ -635,7 +673,7 @@ class BioSeqDatabase:
 
         Returns the number of records loaded.
         """
-        db_loader = Loader.DatabaseLoader(self.adaptor, self.dbid, \
+        db_loader = Loader.DatabaseLoader(self.adaptor, self.dbid,
                                           fetch_NCBI_taxonomy)
         num_records = 0
         global _POSTGRES_RULES_PRESENT
@@ -661,7 +699,7 @@ class BioSeqDatabase:
                       "'%s' AND version = '%s' AND biodatabase_id = '%s')"
                 self.adaptor.execute(sql % (gi, self.dbid, accession, version, self.dbid))
                 if self.adaptor.cursor.fetchone():
-                    raise self.adaptor.conn.IntegrityError("Duplicate record " 
+                    raise self.adaptor.conn.IntegrityError("Duplicate record "
                                      "detected: record has not been inserted")
             #End of hack
             db_loader.load_seqrecord(cur_record)
