@@ -49,6 +49,7 @@ def is_pypy():
         pass
     return False
 
+
 def is_numpy():
     if is_pypy():
         return False
@@ -76,15 +77,33 @@ DOCTEST_MODULES = [
                    "Bio.AlignIO.StockholmIO",
                    "Bio.Alphabet",
                    "Bio.Application",
+                   "Bio.bgzf",
                    "Bio.Blast.Applications",
                    "Bio.Emboss.Applications",
                    "Bio.GenBank",
                    "Bio.KEGG.Compound",
                    "Bio.KEGG.Enzyme",
                    "Bio.Motif",
+                   "Bio.Motif.Applications._AlignAce",
+                   "Bio.Motif.Applications._XXmotif",
+                   "Bio.motifs",
+                   "Bio.motifs.applications._alignace",
+                   "Bio.motifs.applications._xxmotif",
                    "Bio.pairwise2",
+                   "Bio.Phylo.Applications._Raxml",
+                   "Bio.SearchIO",
+                   "Bio.SearchIO._model",
+                   "Bio.SearchIO._model.query",
+                   "Bio.SearchIO._model.hit",
+                   "Bio.SearchIO._model.hsp",
+                   "Bio.SearchIO.BlastIO",
+                   "Bio.SearchIO.HmmerIO",
+                   "Bio.SearchIO.FastaIO",
+                   "Bio.SearchIO.BlatIO",
+                   "Bio.SearchIO.ExonerateIO",
                    "Bio.Seq",
                    "Bio.SeqIO",
+                   "Bio.SeqIO.FastaIO",
                    "Bio.SeqIO.AceIO",
                    "Bio.SeqIO.PhdIO",
                    "Bio.SeqIO.QualityIO",
@@ -94,6 +113,7 @@ DOCTEST_MODULES = [
                    "Bio.SeqUtils",
                    "Bio.SeqUtils.MeltingTemp",
                    "Bio.Sequencing.Applications._Novoalign",
+                   "Bio.Sequencing.Applications._bwa",
                    "Bio.Wise",
                    "Bio.Wise.psw",
                   ]
@@ -111,12 +131,52 @@ try:
 except ImportError:
     #Missing on Jython or Python 2.4
     DOCTEST_MODULES.remove("Bio.SeqIO")
+    DOCTEST_MODULES.remove("Bio.SearchIO")
 
 #Skip Bio.Seq doctest under Python 3, see http://bugs.python.org/issue7490
 if sys.version_info[0] == 3:
     DOCTEST_MODULES.remove("Bio.Seq")
 
-system_lang = os.environ.get('LANG', 'C') #Cache this
+#Skip Bio.bgzf doctest for broken gzip, see http://bugs.python.org/issue17666
+def _have_bug17666():
+    """Debug function to check if Python's gzip is broken (PRIVATE).
+
+    Checks for http://bugs.python.org/issue17666 expected in Python 2.7.4,
+    3.2.4 and 3.3.1 only.
+    """
+    import gzip
+    try:
+        from io import BytesIO
+    except ImportError:
+        #Python 2.5 fall back
+        from StringIO import StringIO as BytesIO
+    #Would like to use byte literal here:
+    bgzf_eof = "\x1f\x8b\x08\x04\x00\x00\x00\x00\x00\xff\x06\x00BC" + \
+               "\x02\x00\x1b\x00\x03\x00\x00\x00\x00\x00\x00\x00\x00\x00"
+    if sys.version_info[0] >= 3:
+        import codecs
+        bgzf_eof = codecs.latin_1_encode(bgzf_eof)[0]
+    h = gzip.GzipFile(fileobj=BytesIO(bgzf_eof))
+    try:
+        data = h.read()
+        h.close()
+        assert not data, "Should be zero length, not %i" % len(data)
+        return False
+    except TypeError, err:
+        #TypeError: integer argument expected, got 'tuple'
+        h.close()
+        return True
+if _have_bug17666():
+    DOCTEST_MODULES.remove("Bio.bgzf")
+
+#HACK: Since Python2.5 under Windows have slightly different str(float) output,
+#we're removing doctests that may fail because of this
+if sys.platform == "win32" and sys.version_info < (2,6):
+    DOCTEST_MODULES.remove("Bio.SearchIO._model.hit")
+    DOCTEST_MODULES.remove("Bio.SearchIO._model.hsp")
+
+system_lang = os.environ.get('LANG', 'C')  # Cache this
+
 
 def main(argv):
     """Run tests, return number of failures (integer)."""
@@ -141,7 +201,7 @@ def main(argv):
     # HOWEVER, we do not want to change the default encoding which is
     # rather important on Python 3 with unicode.
     #lang = os.environ['LANG']
-    
+
     # get the command line options
     try:
         opts, args = getopt.getopt(argv, 'gv', ["generate", "verbose",
@@ -239,7 +299,7 @@ class ComparisonTestCase(unittest.TestCase):
 
         if expected_test != self.name:
             expected.close()
-            raise ValueError("\nOutput:   %s\nExpected: %s" \
+            raise ValueError("\nOutput:   %s\nExpected: %s"
                   % (self.name, expected_test))
 
         # now loop through the output and compare it to the expected file
@@ -266,7 +326,7 @@ class ComparisonTestCase(unittest.TestCase):
             # otherwise make sure the two lines are the same
             elif expected_line != output_line:
                 expected.close()
-                raise ValueError("\nOutput  : %s\nExpected: %s" \
+                raise ValueError("\nOutput  : %s\nExpected: %s"
                       % (repr(output_line), repr(expected_line)))
         expected.close()
 
@@ -347,7 +407,7 @@ class TestRunner(unittest.TextTestRunner):
                 sys.stderr.write("%s docstring test ... " % name)
                 #Can't use fromlist=name.split(".") until python 2.5+
                 module = __import__(name, None, None, name.split("."))
-                suite = doctest.DocTestSuite(module)
+                suite = doctest.DocTestSuite(module, optionflags=doctest.ELLIPSIS)
                 del module
             suite.run(result)
             if cur_dir != os.path.abspath("."):
