@@ -5,6 +5,7 @@
 
 import os
 
+from Bio import BiopythonWarning
 from Bio import SeqIO
 from Bio import AlignIO
 from Bio.SeqRecord import SeqRecord
@@ -20,20 +21,20 @@ except ImportError:
     BytesIO = StringIO
 
 import warnings
-def send_warnings_to_stdout(message, category, filename, lineno,
-                                file=None, line=None):
-    #TODO - Have Biopython DataLossWarning?
-    if category in [UserWarning]:
-        print "%s - %s" % (category.__name__, message)
-warnings.showwarning = send_warnings_to_stdout
 
+# TODO - Convert this to using unittest, and check desired warnings
+# are issued. Used to do that by capturing warnings to stdout and
+# verifying via the print-and-compare check. However, there was some
+# frustrating cross-platform inconsistency I couldn't resolve.
+warnings.simplefilter('ignore', BiopythonWarning)
 
 protein_alphas = [Alphabet.generic_protein]
 dna_alphas = [Alphabet.generic_dna]
 rna_alphas = [Alphabet.generic_rna]
 nucleotide_alphas = [Alphabet.generic_nucleotide,
                      Alphabet.Gapped(Alphabet.generic_nucleotide)]
-no_alpha_formats = ["fasta","clustal","phylip","phylip-relaxed","tab","ig",
+no_alpha_formats = ["fasta","clustal","phylip","phylip-relaxed",
+                    "phylip-sequential","tab","ig",
                     "stockholm","emboss", "fastq","fastq-solexa",
                     "fastq-illumina","qual"]
 possible_unknown_seq_formats = ["qual", "genbank", "gb", "embl", "imgt"]
@@ -49,8 +50,8 @@ for format in sorted(SeqIO._FormatToWriter):
 for format in sorted(AlignIO._FormatToWriter):
     if format not in test_write_read_alignment_formats:
         test_write_read_alignment_formats.append(format)
-test_write_read_alignment_formats.remove("gb") #an alias for genbank
-test_write_read_alignment_formats.remove("fastq-sanger") #an alias for fastq
+test_write_read_alignment_formats.remove("gb")  # an alias for genbank
+test_write_read_alignment_formats.remove("fastq-sanger")  # an alias for fastq
 
 # test_files is a list of tuples containing:
 # - string:  file format
@@ -58,7 +59,7 @@ test_write_read_alignment_formats.remove("fastq-sanger") #an alias for fastq
 # - string:  relative filename
 # - integer: number of sequences
 
-test_files = [ \
+test_files = [
     ("sff",    False, 'Roche/E3MFGYR02_random_10_reads.sff', 10),
 #Following examples are also used in test_Clustalw.py
     ("clustal",True,  'Clustalw/cw02.aln', 2),
@@ -79,22 +80,22 @@ test_files = [ \
     ("fasta",  False, 'Fasta/rose.pro', 1),
     ("fasta",  False, 'Fasta/rosemary.pro', 1),
 #Following examples are also used in test_BioSQL_SeqIO.py
-    ("fasta",  False, 'Fasta/f001', 1), #Protein
-    ("fasta",  False, 'Fasta/f002', 3), #DNA
-    #("fasta", False, 'Fasta/f003', 2), #Protein with comments
-    ("fasta",  False, 'Fasta/fa01', 2), #Protein with gaps
+    ("fasta",  False, 'Fasta/f001', 1),  # Protein
+    ("fasta",  False, 'Fasta/f002', 3),  # DNA
+    #("fasta", False, 'Fasta/f003', 2),  # Protein with comments
+    ("fasta",  False, 'Fasta/fa01', 2),  # Protein with gaps
 #Following are also used in test_SeqIO_features.py, see also NC_005816.gb
     ("fasta",  False, 'GenBank/NC_005816.fna', 1),
     ("fasta",  False, 'GenBank/NC_005816.ffn', 10),
     ("fasta",  False, 'GenBank/NC_005816.faa', 10),
     ("fasta",  False, 'GenBank/NC_000932.faa', 85),
-    ("tab",  False, 'GenBank/NC_005816.tsv', 10), # FASTA -> Tabbed
+    ("tab",  False, 'GenBank/NC_005816.tsv', 10),  # FASTA -> Tabbed
 #Following examples are also used in test_GFF.py
-    ("fasta",  False, 'GFF/NC_001802.fna', 1), #upper case
-    ("fasta",  False, 'GFF/NC_001802lc.fna', 1), #lower case
-    ("fasta",  True,  'GFF/multi.fna', 3), #Trivial nucleotide alignment
+    ("fasta",  False, 'GFF/NC_001802.fna', 1),  # upper case
+    ("fasta",  False, 'GFF/NC_001802lc.fna', 1),  # lower case
+    ("fasta",  True,  'GFF/multi.fna', 3),  # Trivial nucleotide alignment
 #Following example is also used in test_registry.py
-    ("fasta",  False, 'Registry/seqs.fasta', 2), #contains blank line
+    ("fasta",  False, 'Registry/seqs.fasta', 2),  # contains blank line
 #Following example is also used in test_Nexus.py
     ("nexus",  True,  'Nexus/test_Nexus_input.nex', 9),
 #Following examples are also used in test_SwissProt.py
@@ -127,36 +128,40 @@ test_files = [ \
     ("genbank",False, 'GenBank/iro.gb', 1),
     ("genbank",False, 'GenBank/pri1.gb', 1),
     ("genbank",False, 'GenBank/arab1.gb', 1),
-    ("genbank",False, 'GenBank/protein_refseq.gb', 1), #Old version
-    ("genbank",False, 'GenBank/protein_refseq2.gb', 1), #Revised version
+    ("genbank",False, 'GenBank/protein_refseq.gb', 1),  # Old version
+    ("genbank",False, 'GenBank/protein_refseq2.gb', 1),  # Revised version
     ("genbank",False, 'GenBank/extra_keywords.gb', 1),
     ("genbank",False, 'GenBank/one_of.gb', 1),
-    ("genbank",False, 'GenBank/NT_019265.gb', 1), #contig, no sequence
+    ("genbank",False, 'GenBank/NT_019265.gb', 1),  # contig, no sequence
     ("genbank",False, 'GenBank/origin_line.gb', 1),
     ("genbank",False, 'GenBank/blank_seq.gb', 1),
     ("genbank",False, 'GenBank/dbsource_wrap.gb', 1),
-    ("genbank",False, 'GenBank/NC_005816.gb', 1), #See also AE017046.embl
+    ("genbank",False, 'GenBank/NC_005816.gb', 1),  # See also AE017046.embl
     ("genbank",False, 'GenBank/NC_000932.gb', 1),
-    ("genbank",False, 'GenBank/pBAD30.gb', 1), #Odd LOCUS line from Vector NTI
+    ("genbank",False, 'GenBank/pBAD30.gb', 1),  # Odd LOCUS line from Vector NTI
 # The next example is a truncated copy of gbvrl1.seq from
 # ftp://ftp.ncbi.nih.gov/genbank/gbvrl1.seq.gz
 # This includes an NCBI header, and the first three records:
     ("genbank",False, 'GenBank/gbvrl1_start.seq', 3),
 #Following files are also used in test_GFF.py
     ("genbank",False, 'GFF/NC_001422.gbk', 1),
+#Generated with Entrez.efetch("protein", id="16130152", rettype="gbwithparts")
+    ("genbank",False, 'GenBank/NP_416719.gbwithparts', 1),
 #Following files are currently only used here or in test_SeqIO_index.py:
-    ("embl",   False, 'EMBL/epo_prt_selection.embl', 9), #proteins
+    ("embl",   False, 'EMBL/epo_prt_selection.embl', 9),  # proteins
     ("embl",   False, 'EMBL/TRBG361.embl', 1),
     ("embl",   False, 'EMBL/DD231055_edited.embl', 1),
-    ("embl",   False, 'EMBL/SC10H5.embl', 1), # Pre 2006 style ID line
-    ("embl",   False, 'EMBL/U87107.embl', 1), # Old ID line with SV line
-    ("embl",   False, 'EMBL/AAA03323.embl', 1), # 2008, PA line but no AC
-    ("embl",   False, 'EMBL/AE017046.embl', 1), #See also NC_005816.gb
-    ("embl",   False, 'EMBL/Human_contigs.embl', 2), #contigs, no sequences
-    ("embl",   False, 'EMBL/A04195.imgt', 1), # features over indented for EMBL
-    ("imgt",   False, 'EMBL/A04195.imgt', 1), # features over indented for EMBL
+    ("embl",   False, 'EMBL/DD231055_edited2.embl', 1), #Partial ID line
+    ("embl",   False, 'EMBL/SC10H5.embl', 1),  # Pre 2006 style ID line
+    ("embl",   False, 'EMBL/U87107.embl', 1),  # Old ID line with SV line
+    ("embl",   False, 'EMBL/AAA03323.embl', 1),  # 2008, PA line but no AC
+    ("embl",   False, 'EMBL/AE017046.embl', 1),  # See also NC_005816.gb
+    ("embl",   False, 'EMBL/Human_contigs.embl', 2),  # contigs, no sequences
+    ("embl",   False, 'EMBL/location_wrap.embl', 1),  # wrapped locations and unspecified type
+    ("embl",   False, 'EMBL/A04195.imgt', 1),  # features over indented for EMBL
+    ("imgt",   False, 'EMBL/A04195.imgt', 1),  # features over indented for EMBL
     ("stockholm", True,  'Stockholm/simple.sth', 2),
-    ("stockholm", True,  'Stockholm/funny.sth', 5),
+    ("stockholm", True,  'Stockholm/funny.sth', 6),
 #Following PHYLIP files are currently only used here and in test_AlignIO.py,
 #and are mostly from Joseph Felsenstein's PHYLIP v3.6 documentation:
     ("phylip", True,  'Phylip/reference_dna.phy', 6),
@@ -212,6 +217,7 @@ test_files = [ \
     ("abi", False, 'Abi/3730.ab1', 1),
     ]
 
+
 class ForwardOnlyHandle(object):
     """Mimic a network handle without seek and tell methods etc."""
     def __init__(self, handle):
@@ -232,6 +238,7 @@ class ForwardOnlyHandle(object):
     def close(self):
         return self._handle.close()
 
+
 def compare_record(record_one, record_two):
     """This is meant to be a strict comparison for exact agreement..."""
     assert isinstance(record_one, SeqRecord)
@@ -252,16 +259,17 @@ def compare_record(record_one, record_two):
         #object (out of heap memory error)
         if record_one.seq._character != record_two.seq._character:
             return False
-    elif record_one.seq.tostring() != record_two.seq.tostring():
+    elif str(record_one.seq) != str(record_two.seq):
         return False
     #TODO - check features and annotation (see code for BioSQL tests)
-    for key in set(record_one.letter_annotations).intersection( \
+    for key in set(record_one.letter_annotations).intersection(
                    record_two.letter_annotations):
         if record_one.letter_annotations[key] != \
            record_two.letter_annotations[key]:
             return False
     return True
 
+
 def record_summary(record, indent=" "):
     """Returns a concise summary of a SeqRecord object as a string"""
     if record.id == record.name:
@@ -272,31 +280,33 @@ def record_summary(record, indent=" "):
         answer += "None"
     else:
         if len(record.seq) > 50:
-            answer += record.seq[:40].tostring() + "..." + record.seq[-7:].tostring()
+            answer += str(record.seq[:40]) + "..." + str(record.seq[-7:])
         else:
-            answer += record.seq.tostring()
+            answer += str(record.seq)
         answer += "', length=%i" % (len(record.seq))
     return answer
 
+
 def col_summary(col_text):
     if len(col_text) < 65:
         return col_text
     else:
         return col_text[:60] + "..." + col_text[-5:]
 
+
 def alignment_summary(alignment, index=" "):
     """Returns a concise summary of an Alignment object as a string"""
     answer = []
     alignment_len = alignment.get_alignment_length()
     rec_count = len(alignment)
     for i in range(min(5,alignment_len)):
-        answer.append(index + col_summary(alignment.get_column(i)) \
+        answer.append(index + col_summary(alignment.get_column(i))
                             + " alignment column %i" % i)
     if alignment_len > 5:
         i = alignment_len - 1
-        answer.append(index + col_summary("|" * rec_count) \
+        answer.append(index + col_summary("|" * rec_count)
                             + " ...")
-        answer.append(index + col_summary(alignment.get_column(i)) \
+        answer.append(index + col_summary(alignment.get_column(i))
                             + " alignment column %i" % i)
     return "\n".join(answer)
 
@@ -307,9 +317,9 @@ def check_simple_write_read(records, indent=" "):
         if format not in possible_unknown_seq_formats \
         and isinstance(records[0].seq, UnknownSeq) \
         and len(records[0].seq) > 100:
-           #Skipping for speed.  Some of the unknown sequences are
-           #rather long, and it seems a bit pointless to record them.
-           continue
+            #Skipping for speed.  Some of the unknown sequences are
+            #rather long, and it seems a bit pointless to record them.
+            continue
         print indent+"Checking can write/read as '%s' format" % format
 
         #Going to write to a handle...
@@ -337,8 +347,9 @@ def check_simple_write_read(records, indent=" "):
                 print "Failed: Probably len() of None"
             else:
                 print indent+"Failed: %s" % str(e)
-            assert format != t_format, \
-                   "Should be able to re-write in the original format!"
+            if records[0].seq.alphabet.letters is not None:
+                assert format != t_format, \
+                       "Should be able to re-write in the original format!"
             #Carry on to the next format:
             continue
 
@@ -352,7 +363,7 @@ def check_simple_write_read(records, indent=" "):
             #I want to see the output when called from the test harness,
             #run_tests.py (which can be funny about new lines on Windows)
             handle.seek(0)
-            raise ValueError("%s\n\n%s\n\n%s" \
+            raise ValueError("%s\n\n%s\n\n%s"
                               % (str(e), repr(handle.read()), repr(records)))
 
         assert len(records2) == t_count
@@ -370,15 +381,15 @@ def check_simple_write_read(records, indent=" "):
                     #UnknownSeq object (out of heap memory error)
                     assert r1.seq._character.upper() == r2.seq._character
                 else:
-                    assert r1.seq.tostring().upper() == r2.seq.tostring()
+                    assert str(r1.seq).upper() == str(r2.seq)
             elif format == "qual":
                 assert isinstance(r2.seq, UnknownSeq)
                 assert len(r2) == len(r1)
             else:
-                assert r1.seq.tostring() == r2.seq.tostring()
+                assert str(r1.seq) == str(r2.seq)
             #Beware of different quirks and limitations in the
             #valid character sets and the identifier lengths!
-            if format=="phylip":
+            if format in ["phylip", "phylip-sequential"]:
                 assert r1.id.replace("[","").replace("]","")[:10] == r2.id, \
                        "'%s' vs '%s'" % (r1.id, r2.id)
             elif format=="phylip-relaxed":
@@ -408,7 +419,8 @@ def check_simple_write_read(records, indent=" "):
 
 #Check parsers can cope with an empty file
 for t_format in SeqIO._FormatToIterator:
-    if t_format in SeqIO._BinaryFormats or t_format=="uniprot-xml":
+    if t_format in SeqIO._BinaryFormats or \
+       t_format in ("uniprot-xml", "pdb-seqres", "pdb-atom"):
         #Not allowed empty SFF files.
         continue
     handle = StringIO()
@@ -426,9 +438,9 @@ for (t_format, t_alignment, t_filename, t_count) in test_files:
 
     #Try as an iterator using handle
     h = open(t_filename,mode)
-    records  = list(SeqIO.parse(handle=h, format=t_format))
+    records = list(SeqIO.parse(handle=h, format=t_format))
     h.close()
-    assert len(records)  == t_count, \
+    assert len(records) == t_count, \
          "Found %i records but expected %i" % (len(records), t_count)
 
     #Try using the iterator with a for loop, and a filename not handle
@@ -515,7 +527,6 @@ for (t_format, t_alignment, t_filename, t_count) in test_files:
         assert len(record.dbxrefs) == len(record.dbxrefs), \
                "Repeated cross reference in dbxrefs: %s" % repr(record.dbxrefs)
 
-
         #Check the lists obtained by the different methods agree
         assert compare_record(record, records2[i])
         assert compare_record(record, records3[i])
@@ -532,11 +543,11 @@ for (t_format, t_alignment, t_filename, t_count) in test_files:
 
     # Check Bio.SeqIO.read(...)
     if t_count == 1:
-        record = SeqIO.read(handle=open(t_filename,mode), format=t_format)
+        record = SeqIO.read(t_filename, format=t_format)
         assert isinstance(record, SeqRecord)
     else:
         try:
-            record = SeqIO.read(open(t_filename), t_format)
+            record = SeqIO.read(t_filename, t_format)
             assert False, "Bio.SeqIO.read(...) should have failed"
         except ValueError:
             #Expected to fail
@@ -545,9 +556,11 @@ for (t_format, t_alignment, t_filename, t_count) in test_files:
     # Check alphabets
     for record in records:
         base_alpha = Alphabet._get_base_alphabet(record.seq.alphabet)
-        assert isinstance(base_alpha, Alphabet.SingleLetterAlphabet)
-        if t_format in no_alpha_formats:
-            assert base_alpha == Alphabet.single_letter_alphabet # Too harsh?
+        if isinstance(base_alpha, Alphabet.SingleLetterAlphabet):
+            if t_format in no_alpha_formats:
+                assert base_alpha == Alphabet.single_letter_alphabet # Too harsh?
+        else:
+            base_alpha = None
     if base_alpha is None:
         good = []
         bad =[]
@@ -572,30 +585,36 @@ for (t_format, t_alignment, t_filename, t_count) in test_files:
     for given_alpha in good:
         #These should all work...
         given_base = Alphabet._get_base_alphabet(given_alpha)
-        for record in SeqIO.parse(open(t_filename,mode),t_format,given_alpha):
+        for record in SeqIO.parse(t_filename,t_format,given_alpha):
             base_alpha = Alphabet._get_base_alphabet(record.seq.alphabet)
             assert isinstance(base_alpha, given_base.__class__)
             assert base_alpha == given_base
         if t_count == 1:
-            record = SeqIO.read(open(t_filename,mode),t_format,given_alpha)
+            h = open(t_filename,mode)
+            record = SeqIO.read(h,t_format,given_alpha)
+            h.close()
             assert isinstance(base_alpha, given_base.__class__)
             assert base_alpha == given_base
     for given_alpha in bad:
         #These should all fail...
+        h = open(t_filename,mode)
         try:
-            print SeqIO.parse(open(t_filename,mode),t_format,given_alpha).next()
+            print SeqIO.parse(h,t_format,given_alpha).next()
+            h.close()
             assert False, "Forcing wrong alphabet, %s, should fail (%s)" \
                    % (repr(given_alpha), t_filename)
         except ValueError:
+            #Good - should fail
             pass
+        h.close()
     del good, bad, given_alpha, base_alpha
 
     if t_alignment:
         print "Testing reading %s format file %s as an alignment" \
               % (t_format, t_filename)
 
-        alignment = MultipleSeqAlignment(SeqIO.parse( \
-                    handle=open(t_filename,mode), format=t_format))
+        alignment = MultipleSeqAlignment(SeqIO.parse(
+                    handle=t_filename, format=t_format))
         assert len(alignment) == t_count
 
         alignment_len = alignment.get_alignment_length()
