@@ -7,21 +7,64 @@
 import unittest
 import sys
 if sys.version_info[0] == 3:
-   maketrans = str.maketrans
+    maketrans = str.maketrans
 else:
-   from string import maketrans
+    from string import maketrans
 
 from Bio.Alphabet import generic_protein, generic_nucleotide, \
                          generic_dna, generic_rna
 from Bio.Alphabet.IUPAC import protein, extended_protein
 from Bio.Alphabet.IUPAC import unambiguous_dna, ambiguous_dna, ambiguous_rna
 from Bio.Data.IUPACData import ambiguous_dna_values, ambiguous_rna_values
-from Bio.Seq import Seq, UnknownSeq, MutableSeq
-from Bio.Data.CodonTable import TranslationError
+from Bio.Seq import Seq, UnknownSeq, MutableSeq, translate
+from Bio.Data.CodonTable import TranslationError, CodonTable
+
+#This is just the standard table with less stop codons
+#(replaced with coding for O as an artifical example)
+special_table = CodonTable(forward_table={
+    'TTT': 'F', 'TTC': 'F', 'TTA': 'L', 'TTG': 'L',
+    'TCT': 'S', 'TCC': 'S', 'TCA': 'S', 'TCG': 'S',
+    'TAT': 'Y', 'TAC': 'Y', 'TAA': 'O',
+    'TGT': 'C', 'TGC': 'C', 'TGA': 'O', 'TGG': 'W',
+    'CTT': 'L', 'CTC': 'L', 'CTA': 'L', 'CTG': 'L',
+    'CCT': 'P', 'CCC': 'P', 'CCA': 'P', 'CCG': 'P',
+    'CAT': 'H', 'CAC': 'H', 'CAA': 'Q', 'CAG': 'Q',
+    'CGT': 'R', 'CGC': 'R', 'CGA': 'R', 'CGG': 'R',
+    'ATT': 'I', 'ATC': 'I', 'ATA': 'I', 'ATG': 'M',
+    'ACT': 'T', 'ACC': 'T', 'ACA': 'T', 'ACG': 'T',
+    'AAT': 'N', 'AAC': 'N', 'AAA': 'K', 'AAG': 'K',
+    'AGT': 'S', 'AGC': 'S', 'AGA': 'R', 'AGG': 'R',
+    'GTT': 'V', 'GTC': 'V', 'GTA': 'V', 'GTG': 'V',
+    'GCT': 'A', 'GCC': 'A', 'GCA': 'A', 'GCG': 'A',
+    'GAT': 'D', 'GAC': 'D', 'GAA': 'E', 'GAG': 'E',
+    'GGT': 'G', 'GGC': 'G', 'GGA': 'G', 'GGG': 'G'},
+    start_codons=['TAA', 'TAG', 'TGA'],
+    stop_codons=['TAG'])
+
+Chilodonella_uncinata_table = CodonTable(forward_table={
+    'TTT': 'F', 'TTC': 'F', 'TTA': 'L', 'TTG': 'L',
+    'TCT': 'S', 'TCC': 'S', 'TCA': 'S', 'TCG': 'S',
+    'TAT': 'Y', 'TAC': 'Y',             'TAG': 'Q',
+    'TGT': 'C', 'TGC': 'C', 'TGA': 'W', 'TGG': 'W',
+    'CTT': 'L', 'CTC': 'L', 'CTA': 'L', 'CTG': 'L',
+    'CCT': 'P', 'CCC': 'P', 'CCA': 'P', 'CCG': 'P',
+    'CAT': 'H', 'CAC': 'H', 'CAA': 'Q', 'CAG': 'Q',
+    'CGT': 'R', 'CGC': 'R', 'CGA': 'R', 'CGG': 'R',
+    'ATT': 'I', 'ATC': 'I', 'ATA': 'I', 'ATG': 'M',
+    'ACT': 'T', 'ACC': 'T', 'ACA': 'T', 'ACG': 'T',
+    'AAT': 'N', 'AAC': 'N', 'AAA': 'K', 'AAG': 'K',
+    'AGT': 'S', 'AGC': 'S', 'AGA': 'R', 'AGG': 'R',
+    'GTT': 'V', 'GTC': 'V', 'GTA': 'V', 'GTG': 'V',
+    'GCT': 'A', 'GCC': 'A', 'GCA': 'A', 'GCG': 'A',
+    'GAT': 'D', 'GAC': 'D', 'GAA': 'E', 'GAG': 'E',
+    'GGT': 'G', 'GGC': 'G', 'GGA': 'G', 'GGG': 'G'},
+    start_codons = [ 'ATG'],
+    stop_codons = ['TAA' ])
 
 
 class StringMethodTests(unittest.TestCase):
-    _examples = [ \
+    _examples = [
+        #These are length 9, a multiple of 3 for translation tests:
         Seq("ACGTGGGGT", generic_protein),
         Seq("ACGTGGGGT", generic_nucleotide),
         Seq("ACGTGGGGT", generic_dna),
@@ -39,22 +82,21 @@ class StringMethodTests(unittest.TestCase):
         UnknownSeq(1, generic_rna),
         UnknownSeq(1, generic_rna, "n"),
         UnknownSeq(1, generic_rna, "N"),
-        UnknownSeq(10, generic_rna, "N"),
-        UnknownSeq(10, generic_dna, "N"),
-        UnknownSeq(10, generic_nucleotide, "N"),
-        UnknownSeq(10, generic_protein, "X"),
-        UnknownSeq(10, character="X"),
-        UnknownSeq(10),
+        UnknownSeq(12, generic_rna, "N"),
+        UnknownSeq(12, generic_dna, "N"),
+        UnknownSeq(12, generic_nucleotide, "N"),
+        UnknownSeq(12, generic_protein, "X"),
+        UnknownSeq(12, character="X"),
+        UnknownSeq(12),
         ]
     for seq in _examples[:]:
         if isinstance(seq, Seq):
             _examples.append(seq.tomutable())
     _start_end_values = [0, 1, 2, 1000, -1, -2, -999]
 
-
     def _test_method(self, method_name, pre_comp_function=None, start_end=False):
         """Check this method matches the plain string's method."""
-        assert isinstance(method_name, str)
+        self.assertTrue(isinstance(method_name, str))
         for example1 in self._examples:
             if not hasattr(example1, method_name):
                 #e.g. MutableSeq does not support find
@@ -73,7 +115,7 @@ class StringMethodTests(unittest.TestCase):
                     i = pre_comp_function(i)
                     j = pre_comp_function(j)
                 if i != j:
-                    raise ValueError("%s.%s(%s) = %i, not %i" \
+                    raise ValueError("%s.%s(%s) = %i, not %i"
                                      % (repr(example1),
                                         method_name,
                                         repr(str2),
@@ -87,7 +129,7 @@ class StringMethodTests(unittest.TestCase):
                         i = pre_comp_function(i)
                         j = pre_comp_function(j)
                     if i != j:
-                        raise ValueError("%s.%s(%s) = %i, not %i" \
+                        raise ValueError("%s.%s(%s) = %i, not %i"
                                          % (repr(example1),
                                             method_name,
                                             repr(example2),
@@ -105,14 +147,14 @@ class StringMethodTests(unittest.TestCase):
                             i = pre_comp_function(i)
                             j = pre_comp_function(j)
                         if i != j:
-                            raise ValueError("%s.%s(%s, %i) = %i, not %i" \
+                            raise ValueError("%s.%s(%s, %i) = %i, not %i"
                                              % (repr(example1),
                                                 method_name,
                                                 repr(str2),
                                                 start,
                                                 i,
                                                 j))
-                        
+
                         for end in self._start_end_values:
                             i = getattr(example1,method_name)(str2, start, end)
                             j = getattr(str1,method_name)(str2, start, end)
@@ -120,7 +162,7 @@ class StringMethodTests(unittest.TestCase):
                                 i = pre_comp_function(i)
                                 j = pre_comp_function(j)
                             if i != j:
-                                raise ValueError("%s.%s(%s, %i, %i) = %i, not %i" \
+                                raise ValueError("%s.%s(%s, %i, %i) = %i, not %i"
                                                  % (repr(example1),
                                                     method_name,
                                                     repr(str2),
@@ -150,24 +192,24 @@ class StringMethodTests(unittest.TestCase):
         except TypeError:
             #Base string only supports this on Python 2.5+, skip this
             return
-        
+
         #Now check with a tuple of sub sequences
         for example1 in self._examples:
             if not hasattr(example1, "startswith"):
                 #e.g. MutableSeq does not support this
                 continue
-            subs = tuple([example1[start:start+2] for start \
+            subs = tuple([example1[start:start+2] for start
                           in range(0, len(example1)-2,3)])
             subs_str = tuple([str(s) for s in subs])
 
             self.assertEqual(str(example1).startswith(subs_str),
                              example1.startswith(subs))
             self.assertEqual(str(example1).startswith(subs_str),
-                             example1.startswith(subs_str)) #strings!
+                             example1.startswith(subs_str))  # strings!
             self.assertEqual(str(example1).startswith(subs_str,3),
                              example1.startswith(subs,3))
             self.assertEqual(str(example1).startswith(subs_str,2,6),
-                             example1.startswith(subs,2,6))        
+                             example1.startswith(subs,2,6))
 
     def test_str_endswith(self):
         """Check matches the python string endswith method."""
@@ -184,14 +226,14 @@ class StringMethodTests(unittest.TestCase):
             if not hasattr(example1, "endswith"):
                 #e.g. MutableSeq does not support this
                 continue
-            subs = tuple([example1[start:start+2] for start \
+            subs = tuple([example1[start:start+2] for start
                           in range(0, len(example1)-2,3)])
             subs_str = tuple([str(s) for s in subs])
 
             self.assertEqual(str(example1).endswith(subs_str),
                              example1.endswith(subs))
             self.assertEqual(str(example1).startswith(subs_str),
-                             example1.startswith(subs_str)) #strings!
+                             example1.startswith(subs_str))  # strings!
             self.assertEqual(str(example1).endswith(subs_str,3),
                              example1.endswith(subs,3))
             self.assertEqual(str(example1).endswith(subs_str,2,6),
@@ -232,14 +274,16 @@ class StringMethodTests(unittest.TestCase):
     def test_str_upper(self):
         """Check matches the python string upper method."""
         for example1 in self._examples:
-            if isinstance(example1, MutableSeq) : continue
+            if isinstance(example1, MutableSeq):
+                continue
             str1 = str(example1)
             self.assertEqual(str(example1.upper()), str1.upper())
 
-    def test_str_upper(self):
+    def test_str_lower(self):
         """Check matches the python string lower method."""
         for example1 in self._examples:
-            if isinstance(example1, MutableSeq) : continue
+            if isinstance(example1, MutableSeq):
+                continue
             str1 = str(example1)
             self.assertEqual(str(example1.lower()), str1.lower())
 
@@ -258,11 +302,11 @@ class StringMethodTests(unittest.TestCase):
                         if step == 0:
                             try:
                                 print example1[i:j:step]
-                                self._assert(False) #Should fail!
+                                self._assert(False)  # Should fail!
                             except ValueError:
                                 pass
                         else:
-                            self.assertEqual(str(example1[i:j:step]), \
+                            self.assertEqual(str(example1[i:j:step]),
                                              str1[i:j:step])
 
     def test_tostring(self):
@@ -274,7 +318,8 @@ class StringMethodTests(unittest.TestCase):
     def test_tomutable(self):
         """Check obj.tomutable() method."""
         for example1 in self._examples:
-            if isinstance(example1, MutableSeq) : continue
+            if isinstance(example1, MutableSeq):
+                continue
             mut = example1.tomutable()
             self.assertTrue(isinstance(mut, MutableSeq))
             self.assertEqual(str(mut), str(example1))
@@ -296,7 +341,8 @@ class StringMethodTests(unittest.TestCase):
         """Check obj.complement() method."""
         mapping = ""
         for example1 in self._examples:
-            if isinstance(example1, MutableSeq) : continue
+            if isinstance(example1, MutableSeq):
+                continue
             try :
                 comp = example1.complement()
             except ValueError, e:
@@ -315,16 +361,16 @@ class StringMethodTests(unittest.TestCase):
                 mapping = maketrans("CGcg","GCgc")
             else :
                 #TODO - look at alphabet?
-                assert False, example1
-                continue
+                raise ValueError(example1)
             self.assertEqual(str1.translate(mapping), str(comp))
             self.assertEqual(comp.alphabet, example1.alphabet)
-                
+
     def test_the_reverse_complement(self):
         """Check obj.reverse_complement() method."""
         mapping = ""
         for example1 in self._examples:
-            if isinstance(example1, MutableSeq) : continue
+            if isinstance(example1, MutableSeq):
+                continue
             try :
                 comp = example1.reverse_complement()
             except ValueError, e:
@@ -351,41 +397,55 @@ class StringMethodTests(unittest.TestCase):
             """Check obj.transcribe() method."""
             mapping = ""
             for example1 in self._examples:
-                if isinstance(example1, MutableSeq) : continue
+                if isinstance(example1, MutableSeq):
+                    continue
                 try :
                     tran = example1.transcribe()
                 except ValueError, e:
-                    if str(e) == "Proteins cannot be transcribed!" : continue
-                    if str(e) == "RNA cannot be transcribed!" : continue
+                    if str(e) == "Proteins cannot be transcribed!":
+                        continue
+                    if str(e) == "RNA cannot be transcribed!":
+                        continue
                     raise e
                 str1 = str(example1)
+                if len(str1) % 3 != 0:
+                    #TODO - Check for or silence the expected warning?
+                    continue
                 self.assertEqual(str1.replace("T","U").replace("t","u"), str(tran))
-                self.assertEqual(tran.alphabet, generic_rna) #based on limited examples             
+                self.assertEqual(tran.alphabet, generic_rna)  # based on limited examples
 
     def test_the_back_transcription(self):
             """Check obj.back_transcribe() method."""
             mapping = ""
             for example1 in self._examples:
-                if isinstance(example1, MutableSeq) : continue
+                if isinstance(example1, MutableSeq):
+                    continue
                 try :
                     tran = example1.back_transcribe()
                 except ValueError, e:
-                    if str(e) == "Proteins cannot be back transcribed!" : continue
-                    if str(e) == "DNA cannot be back transcribed!" : continue
+                    if str(e) == "Proteins cannot be back transcribed!":
+                        continue
+                    if str(e) == "DNA cannot be back transcribed!":
+                        continue
                     raise e
                 str1 = str(example1)
                 self.assertEqual(str1.replace("U","T").replace("u","t"), str(tran))
-                self.assertEqual(tran.alphabet, generic_dna) #based on limited examples             
+                self.assertEqual(tran.alphabet, generic_dna)  # based on limited examples
 
     def test_the_translate(self):
             """Check obj.translate() method."""
             mapping = ""
             for example1 in self._examples:
-                if isinstance(example1, MutableSeq) : continue
+                if isinstance(example1, MutableSeq):
+                    continue
+                if len(example1) % 3 != 0:
+                    #TODO - Check for or silence the expected warning?
+                    continue
                 try :
                     tran = example1.translate()
                 except ValueError, e:
-                    if str(e) == "Proteins cannot be translated!" : continue
+                    if str(e) == "Proteins cannot be translated!":
+                        continue
                     raise e
                 #This is based on the limited example not having stop codons:
                 if tran.alphabet not in [extended_protein, protein, generic_protein]:
@@ -409,8 +469,19 @@ class StringMethodTests(unittest.TestCase):
             self.assertEqual("**WSS", str(nuc.translate(table=9)))
             self.assertEqual("**CRR", str(nuc.translate(table='Euplotid Nuclear')))
             self.assertEqual("***RR", str(nuc.translate(table=11)))
+            self.assertEqual("***RR", str(nuc.translate(table='11')))
             self.assertEqual("***RR", str(nuc.translate(table='Bacterial')))
             self.assertEqual("", str(nuc.translate(to_stop=True)))
+            self.assertEqual("O*ORR", str(nuc.translate(table=special_table)))
+            self.assertEqual("*QWRR", str(nuc.translate(table=Chilodonella_uncinata_table)))
+            #These test the Bio.Seq.translate() function - move these?:
+            self.assertEqual("*QWRR", translate(str(nuc), table=Chilodonella_uncinata_table))
+            self.assertEqual("O*ORR", translate(str(nuc), table=special_table))
+            self.assertEqual("", translate(str(nuc), to_stop=True))
+            self.assertEqual("***RR", translate(str(nuc), table='Bacterial'))
+            self.assertEqual("***RR", translate(str(nuc), table='11'))
+            self.assertEqual("***RR", translate(str(nuc), table=11))
+            self.assertEqual("**W**", translate(str(nuc), table=2))
         self.assertEqual(str(Seq("TAT").translate()), "Y")
         self.assertEqual(str(Seq("TAR").translate()), "*")
         self.assertEqual(str(Seq("TAN").translate()), "X")
@@ -424,7 +495,6 @@ class StringMethodTests(unittest.TestCase):
         self.assertEqual(str(Seq("tan").translate()), "X")
         self.assertEqual(str(Seq("nnn").translate()), "X")
 
-
     def test_the_translation_of_invalid_codons(self):
         """Check obj.translate() method with invalid codons."""
         for codon in ["TA?", "N-N", "AC_", "Ac_"]:
@@ -446,16 +516,16 @@ class StringMethodTests(unittest.TestCase):
             for c1 in ambig:
                 for c2 in ambig:
                     for c3 in ambig:
-                        values = set([str(Seq(a+b+c).translate()) \
-                                      for a in ambig_values[c1] \
-                                      for b in ambig_values[c2] \
+                        values = set([str(Seq(a+b+c).translate())
+                                      for a in ambig_values[c1]
+                                      for b in ambig_values[c2]
                                       for c in ambig_values[c3]])
                         t = str(Seq(c1+c2+c3).translate())
                         if t=="*":
                             self.assertEqual(values, set("*"))
                         elif t=="X":
-                            self.assertTrue(len(values) > 1, \
-                                "translate('%s') = '%s' not '%s'" \
+                            self.assertTrue(len(values) > 1,
+                                "translate('%s') = '%s' not '%s'"
                                 % (c1+c2+c3, t, ",".join(values)))
                         elif t=="Z":
                             self.assertEqual(values, set("EQ"))
